      program edi2ts

c     converts time series in EDI format to TS format

      implicit none

      real MISDAT
      parameter ( MISDAT = 99999.9 )

      real lat, lon
      real xlatdeg, xlatmin, xlatsec, xlondeg, xlonmin, xlonsec
      real delta
      character stat*6, fin*20, fout*20,line*80
      real ex(100000), ey(100000), hx(100000), hy(100000), hz(100000),
     &     rx(100000), ry(100000)
      integer i, npts


c...open input file
      fin = 'doorn2_low.edi'
      call tin( 'Give EDI file', fin )
      open(unit=4,file=fin, status='OLD', err=9999)


c...open output file
      stat = fin(1:6)
      call tin( 'Give output station name', stat )

      fout = stat//'a1.ts'
      call tin('Give output ts filename', fout )
      write(*,*)'Opening output ts file >', fout
      open(unit=7,file=fout,form='unformatted')

c...get info from EDI file

c...latitude
      read(4,'(a)') line
      do while(line(1:3).ne.'LAT')
        read(4,'(a)') line
      enddo
      line = line(index(line,'=')+1:)
      read(line(:index(line,':')-1), *) xlatdeg
      line = line(index(line,':')+1:)
      read(line(:index(line,':')-1), *) xlatmin
      line = line(index(line,':')+1:)
      read(line, *) xlatsec
      lat = abs(xlatdeg) + xlatmin/60. + xlatsec/3600.
      if( xlatdeg.lt.0. ) lat = -lat
      write(*,*)'Latitude =>', lat

c...longitude
      read(4,'(a)') line
      do while(line(1:3).ne.'LON')
        read(4,'(a)') line
      enddo
      line = line(index(line,'=')+1:)
      read(line(:index(line,':')-1), *) xlondeg
      line = line(index(line,':')+1:)
      read(line(:index(line,':')-1), *) xlonmin
      line = line(index(line,':')+1:)
      read(line, *) xlonsec
      lon = abs(xlondeg) + xlonmin/60. + xlonsec/3600.
      if( xlondeg.lt.0. ) lon = -lon
      write(*,*)'Longitude =>', lon

c...npts
      read(4,'(a)') line
      do while(line(1:5).ne.'NPTS=')
        read(4,'(a)') line
      enddo
      read(line(6:),*) npts


c...delta
      read(4,'(a)') line
      do while(line(1:3).ne.'SR=')
        read(4,'(a)') line
      enddo
      read(line(4:),*) delta
      write(*,*)'Delta (Hz) =', delta
      delta = 1./delta


c...write comment block
      line = '#'
      write(7) line
      line = '# Time series file:'
      write(7) line
      line = '#'
      write(7) line
      line = '# EDI input file: '//fin
      write(7) line
      line = '#'

c...write information block
      line = '>STATION   :'//stat
      write(7) line
      line = '>INSTRUMENT:'
      write(line(13:20),'(i8.3)') 0
      write(7) line
      line = '>WINDOW    :'//fout(1:8)
      write(7) line
      line = '>LATITUDE  :'
      write(line(13:20),'(f8.4)') lat
      write(7) line
      line = '>LONGITUDE :'
      write(line(13:20),'(f8.4)') lon
      write(7) line
      line = '>ELEVATION :'
      write(line(13:20),'(f8.4)') 0.0
      write(7) line

      line = '>COORD_SYS :TRUE NORTH'
      write(7) line

      line = '>FORM      :BINARY'
      write(7) line
      line = '>FORMAT    :UNFORMATTED'
      write(7) line
      line = '>NCHAN     :   5'
      write(7) line

      line = '>CHAN_1    :HX'
      write(7) line
      line = '>SENSOR_1  : 0'
      write(7) line
      line = '>AZIM_1    : 0.'
      write(7) line
      line = '>UNITS_1   :nT'
      write(7) line
      line = '>GAIN_1    :1.0'
      write(7) line

      line = '>CHAN_2    :HY'
      write(7) line
      line = '>SENSOR_2  : 0'
      write(7) line
      line = '>AZIM_2    :90.'
      write(7) line
      line = '>UNITS_2   :nT'
      write(7) line
      line = '>GAIN_2    :1.0'
      write(7) line

      line = '>CHAN_3    :HZ'
      write(7) line
      line = '>SENSOR_3  : 0'
      write(7) line
      line = '>AZIM_3    : 0.'
      write(7) line
      line = '>UNITS_3   :nT'
      write(7) line
      line = '>GAIN_3    :1.0'
      write(7) line

      line = '>CHAN_4    :EX'
      write(7) line
      line = '>SENSOR_4  : 0'
      write(7) line
      line = '>AZIM_4    : 0.'
      write(7) line
      line = '>UNITS_4   :mV/km'
      write(7) line
      line = '>GAIN_4    :1.0'
      write(7) line

      line = '>CHAN_5    :EY'
      write(7) line
      line = '>SENSOR_5  : 0'
      write(7) line
      line = '>AZIM_5    : 0.'
      write(7) line
      line = '>UNITS_5   :mV/km'
      write(7) line
      line = '>GAIN_5    :1.0'
      write(7) line

      line = '>STARTTIME :2000-01-01 00:00:00'
      write(7) line

      line = '>ENDTIME   :2000-12-31 23:59:59'
      write(7) line

      line = '>T_UNITS   :s'
      write(7) line
      line = '>DELTA_T   :'
      write(line(13:),* ) delta
      write(7) line
      line = '>MIS_DATA  :'
      write(line(13:),* ) MISDAT
      write(7) line


c-------------------------------------------------------------------------------
c     read in data

c...move down to line RY=
      read(4,'(a)') line
      do while(line(1:3).ne.'RY=')
        read(4,'(a)') line
      enddo
      write(*,*)'read to RY='


      read(4,*) ex(1)
      do while( ex(1).eq.0. )
        read(4,*) ex(1)
      enddo
      write(*,*)'First value of Ex =', ex(1)
      do i = 2, npts
        read(4,*) ex(i)
      enddo
      write(*,*)'Finished Ex: last =', ex(npts)
      read(4,*) ey(1)
      write(*,*)'First value of Ey =', ey(1)
      do i = 2, npts
        read(4,*) ey(i)
      enddo
      write(*,*)'Finished Ey: last =', ey(npts)
      read(4,*) hx(1)
      write(*,*)'First value of Hx =', hx(1)
      do i = 2, npts
        read(4,*) hx(i)
      enddo
      write(*,*)'Finished Hx: last =', hx(npts)
      read(4,*) hy(1)
      write(*,*)'First value of Hy =', hy(1)
      do i = 2, npts
        read(4,*) hy(i)
      enddo
      write(*,*)'Finished Hy: last =', hy(npts)
      read(4,*) hz(1)
      write(*,*)'First value of Hz =', hz(1)
      do i = 2, npts
        read(4,*) hz(i)
      enddo
      write(*,*)'Finished Hz: last =', hz(npts)



c-------------------------------------------------------------------------------
c     write out data

      do i = 1, npts
        write(7) hx(i)-hx(1), 
     &           hy(i)-hy(1), 
     &           hz(i)-hz(1), 
     &           ex(i)-ex(1), 
     &           ey(i)-ey(1)
      enddo

c-------------------------------------------------------------------------------

      stop


9999  write(*,*)'Could not find file ', fin

      stop
      end
