c=========================================================================
      SUBROUTINE input1d(ind,instr,dat,nent,yy,mm,dd,hh,min,ss,
     >                    tsec,imark)
c  Subroutine to read in LIMS data values 
c     May 13, 1988
c     November 10, 1990:
c       Revised to remove hardware and software gains.
c     June 12, 1991:
c       Revised to read the 6 auxiliary channels.
c
c  August 18, 1992 (Angela Tchen)
c  Separated from parent program file lims_cascade.f to input1d.f
c************************************************************************
c Variables Declared
c
c INTEGERS
c in     - keeps track of # of total entries including amount value in
c       variable bent at first but at end removes bent values from number.
c ind    - data file read unit
c instr  - instrument number to read in
c prterr - to indicate level of error for user
c nchan  - total number of active channels recorded (does not include the 
c       auxiliary channels)
c inc    - calibration file read unit
c bent   - number of non-existent entries between the requested start time
c       and the first valid data entry.
c nent   - number of entries found
c yy     - year
c mm     - month in integer format
c dd     - day in month
c hh     - hour in day (24h format)
c min    - minute
c ss     - second
c tsec   - total # of seconds which then is used to calculate # of entries
c       requested.
c stsec  - # of seconds from the beginning of the year to the start of
c       requested data
c nc     - counter for number of channels identified for setting calibrations
c bstsec - # of seconds from the beginning of the year to the start of window
c byy    - start time year
c bmm    - start time month
c bdd    - start time day
c bhh    - start time hour
c bmin   - start time minute
c bss    - start time second
c idel   - intermediary variable dealing with delta
c nskip  - number of entries to skip
c ntot   - total number of entries requested
c itgain - hardware telluric gain
c lenex  - Ex line length (metres)
c leney  - Ey line length (metres)
c mapcmp(16) - the map for the auxiliary channels
c gain(16)   - the gain setting for different channels in both GROUP 1
c      and GROUP2
c mpchan(5)  - locates the channels in the order required by main program
c ns     - number of seconds between start time of data block and requested
c      start time
c
c INTEGER*2
c dt(16)     - variable where data is first read
c aux1(6)    - auxiliary channel data (every 10 minutes)
c
c REAL
c sc(16)     - is from common block /scales/
c delta      - digitizing interval in Hz.
c dat(16)    - variable for the final form of data 
c cal(5)     - final calibration factor.
c magfac     - magnetometer factor.
c calg(5)    - initial calibration factor read from calibration file.
c calibs(5)  - fine tune calibration from hardware file
c
c CHARACTER
c comp(16)*2 - contains the channel name (ie. Hx, Ey, etc.)  Its subscript
c	corresponds to the actual LIMS channel number.
c
c LOGICAL
c ANS - logical flag for answers to several different questions
c
c************************************************************************
      integer MAXCHAN
      parameter ( MAXCHAN =     5 )

      character cfile*7, comp(16)*2, acomp*2, SCCSID*55

      integer*2 dt(16), aux1(6), intgr(29), filtp1, filtp2
      integer lbox, icall, in, ind, instr, prterr, nchan, inc
      integer bent, nent, yy, mm, dd, hh, min, ss, tsec, ns
      integer stsec, nc, j, m, ic, naux, imark, baseln(3), nauxold
      integer bstsec, byy, bmm, bdd, bhh, bmin, bss, idel, nskip, ntot
      integer itgain, lenex, leney, mapcmp(16), gain(16), mpchan(5)

      logical ANS

      real sc(16), delta, ad2mv, smprt1, smprt2
      real dat(16), cal(5), magfac, calg(5) 
      real calibs(5), cutsl(5), cutsh(5)

      common /scales/ sc
      common /contst/ inc, delta, prterr, nchan
      common /calbtn/ itgain, lenex, leney, magfac, gain, lbox, comp,
     >                mapcmp, cfile
      common /blkst/ byy, bmm, bdd, bhh, bmin, bss
      common /headr2/ smprt1, smprt2, baseln, intgr, filtp1, filtp2
      common /iflag/ icall, cal

      include 'ctrlblk.inc'

      data icall /0/
      data SCCSID /"@(#)input1d1.f     1.2     94/10/18 LIMS\n"/

      icall = icall + 1


      if ( icall .eq. 1 ) then
        in     = 1
        bent   = 0
        nskip  = 0
        nent   = 0
        nc     = 0

c  Display calibration factors required to convert LIMS microvolt data
c     to mV/km and nT.
      if( iprint.ge.1 ) then
      write( *, '(a)') ' The default calibration and converstion values 
     & are:'
      write(*, '(4x,a)') ' Values from RED2BIN are in digitizer units.'
      write(*, '(4x,a)') ' NO gains have been removed.'
      write(*, '(4x,a,i4,a)')' Telluric line lengths are: Ex =',lenex,
     &                       ' m.'
      write(*, '(4x,a,i4,a)')'                            Ey =',leney,
     &                       ' m.'
      write(*, '(4x,a,i4)') ' Instrument telluric gain is = ', itgain
      write(*, '(4x,a,f10.1)') ' Magnetometer conversion factor is', 
     &                           magfac
      write(*, '(10x,a)') ' That is: 10 mV = 1 nT'
      write(*, '(4x,a,a)') ' Electric fields are multiplied by -1 to ',
     &                  'convert from potentials to fields.'
      write(*, '(4x,a,a)') ' The project hardware file fine tunes the ',
     &                  ' gains and contains filter constants.'

        ANS = .TRUE.
        call lin( ' Are these factors correct?', ANS)
        if ( .NOT. ANS ) then
          call iin( ' Give Ex line length in meters', lenex )
          call iin( ' Give Ey line length in meters', leney )
          call iin( ' Give instrument telluric gain', itgain)
          call rin( ' Give magnetometer conversion factor', magfac)
        end if
      end if

c  Read the project hardware file.
c     stime contains the character representation of the data block
c     start time.  Used for finding the correct calibration set.

      rewind( inc )
      call gethardware( inc, instr, nchan, calibs, cutsl, cutsh )


c  The feed to the main program must be:
c                   CHAN NUMBER         COMPONENT     gethardware
c                        1                  Hx             Hx
c                        2                  Hy             Hy
c                        3                  Hz             Ex
c                        4                  Ex             Ey
c                        5                  Ey             Hz

c  MPCHAN locates the channels in the order required by the main program.

c  The subscript of MAPCMP corresponds to the channel number entries 
c     stored in the RED2BIN output file.
c     The value of mapcmp is the actual LIMS channel number.

c  The value of ad2mv (usually 305) converts A/D data into microvolts.
c
      ad2mv = float(intgr(27))

      do 5 j = 1, nchan

        acomp = comp(j)
        call shiftup( acomp )

        if( acomp.eq.'HX' ) then
          nc = nc + 1
          mpchan(1) = nc
          calg(nc) = ad2mv/(magfac*float(gain(j)))
          mapcmp(nc) = j

        else if( acomp.eq.'HY' ) then
          nc = nc + 1
          mpchan(2) = nc
          calg(nc) = ad2mv/(magfac*float(gain(j)))
          mapcmp(nc) = j

        else if( acomp.eq.'HZ' ) then
          nc = nc + 1
          mpchan(3) = nc
          calg(nc) = ad2mv/(magfac*float(gain(j)))
          mapcmp(nc) = j

        else if( acomp.eq.'EX' ) then
          nc = nc + 1
          mpchan(4) = nc
          calg(nc) = ad2mv/float( itgain*lenex*gain(j) )
          mapcmp(nc) = j

        else if( acomp.eq.'EY' ) then
          nc = nc + 1
          mpchan(5) = nc
          calg(nc) = ad2mv/float( itgain*leney*gain(j) )
          mapcmp(nc) = j

        else if( acomp.eq.'GX' ) then
          nc = nc + 1
          mpchan(1) = nc
          calg(nc) = 1./(1000.*float(gain(j)))
          mapcmp(nc) = j

        else if( acomp.eq.'GY' ) then
          nc = nc + 1
          mpchan(2) = nc
          calg(nc) = 1./(1000.*float(gain(j)))
          mapcmp(nc) = j

        else if( acomp.eq.'GZ' ) then
          nc = nc + 1
          mpchan(3) = nc
          calg(nc) = 1./(100.*float(gain(j)))
          mapcmp(nc) = j

        else
          write(*,*)'input1d: ERROR: Unknown channel identifier: ',
     &              'acomp="', acomp,'"'
          stop

        end if

        if( iprint.ge.1 ) then
            write(*, '(a,i2,a,a,a,i2,a,1Pe10.3)' ) 'LIMS channel ',j,
     &     ' (',  comp(j),') assigned LIMSCASB channel ',nc,
     &     ' with calg factor =', calg(nc)
        endif

    5 continue

      if ( nc.ne.nchan ) then
        write(*,*) ' input1d1: WARNING: unidentified channels used.'
        write(*,*) '          Channel calibration factors set to 1.0'
      end if

c  Use the information from the calibration file: note re-ordering
c  of info from "gethardware" (see above re ordering)
      if( nchan.eq.5 ) then
          cal(4) = calg(mpchan(4)) * calibs(3) * (-1.)      ! Ex
          cal(5) = calg(mpchan(5)) * calibs(4) * (-1.)      ! Ey
      end if

          cal(1) = calg(mpchan(1)) * calibs(1)              ! Hx
          cal(2) = calg(mpchan(2)) * calibs(2)              ! Hy
          cal(3) = calg(mpchan(3)) * calibs(5)              ! Hz

      if( iprint.ge.1 ) then
      write(*,'(a)')' j comp(mapcmp(j)) mapcmp(j) gain(mapcmp(j)) mpchan
     &(j)     cal(j) calg(mpchan(j))'
      do 7 j = 1, nchan
      write(*,'(1x,i1,13x,a2,8x,i2,13x,i3,8x,i2,1x,1pe10.3,6x,1pe10.3)')
     &        j, comp(mapcmp(j)), mapcmp(j), gain(mapcmp(j)), mpchan(j),
     &        cal(j), calg(mpchan(j))
    7 continue
      end if

c  Calculate the total number of entries requested.
      if ( delta .ge. 1.0 ) then
        ntot = tsec * ifix ( delta )
      else
        idel =  nint (1.0/delta)
        ntot = tsec/idel
      end if

      if( ntot .le. 0 ) then
        write(*,*)'input1d1: Total number of data entries',
     &            ' requested is 0!!'
        nent = - 1
        go to 9999
      end if

      if( iprint.ge.1 ) then
       write(*,*)'input1d1: Total number of entries requested =', ntot
      end if

 
c  Calculate the start time of the requested data in seconds from beginning
c     of the year.

      call consec ( stsec, yy, mm, dd, hh, min, ss )
      if( iprint.ge.1 ) then
        write(*,'(a,i8,i5,5i3)')'input1d1: Requested start time:'//
     & 'stsec/yy/mm/dd/hh/min/ss   ',stsec,yy,mm,dd,hh,min,ss
      end if

c  Calculate the data block start time in seconds from beginning of the 
c     year.
      call consec ( bstsec, byy, bmm, bdd, bhh, bmin, bss )
      if( iprint.ge.1 ) then
        write(*,'(a,i8,i5,5i3)')'input1d1: Data start time: bstsec/
     &byy/bmm/bdd/bhh/bmin/bss ',bstsec,byy,bmm,bdd,bhh,bmin,bss
      end if

c  Calculate number of seconds between start time of data block and
c     requested start time.
      ns = stsec - bstsec
      if( iprint.ge.1 ) then
      write (*, '(a, i6)')'input1d1: Seconds between requested
     & and data block start time: ns=', ns
      endif

c  
      if( ns.lt.0 ) then
        write(*,'(a,a)')'input1d1: ERROR: requested start time'//
     & 'preceeds first data entry in data block.'
        if ( delta .ge. 1.0 ) then
          bent =  -ns*ifix( delta )
        else
          idel =  nint (1.0/delta)
          bent = -ns/idel
        end if
        write(*,*)' The gap size is ', bent
        goto 9999
      endif

c  Skip data entries to reach the requested start time.
      if ( ns .gt. 0 ) then
        if ( delta .gt. 1.0 ) then
          nskip =  ns*ifix( delta )
        else
          idel =  nint (1.0/delta)
          nskip = ns/idel
        end if

        if( iprint.ge.1 ) then
          write (*,'(a,i8)')'input1d: Number of data entries '//
     &                      'to skip: ',nskip
        end if
 
        do j = 1, nskip
          call readdt(ind,in,naux,delta,nchan,dt,aux1,imark,*8000,*8000)
        end do
      end if          

c  This END IF refers back to line 35 ( if (icall .eq. 1) )
      endif

      nauxold = 1
2     call readdt(ind,in,naux,delta,nchan,dt,aux1,imark,*7000,*9000)
      if( iprint.ge.3 ) then
        write (*, '(1x, i5, 10i7)' )in,( dt(m), m = 1, nchan )
      end if
40    continue

      do 100 ic = 1 ,nchan
        dat(ic) =float(dt(mpchan(ic))) * cal(ic)
100   continue

      if( nauxold .ne. naux ) then
        nauxold = naux
c Auxiliary channel data multiplied by -1 until the Lab gets the 
c polarity sorted out.

c...Ti - instrument temperature
        dat(nchan+4) = -1. * float(aux1(4)) * ad2mv/100000.
c...Th - head       temperature
        dat(nchan+5) = -1. * float(aux1(5)) * ad2mv/100000.
c...Bt - battery voltage
        dat(nchan+6) = -1. * float(aux1(6)) * ad2mv*2./1000000.
      endif
         
      return
9000  continue
      nent = in - bent
      go to 9999

6000  continue
      write(*,'(a,a,a)')'input1d: Cannot locate station ',cfile,
     &                  ' in project calibration file.'
      go to 9999

7000  continue
      write(*,'(a,2i6)')'input1d: Data read error occurred at line:'
     &                  ,in
      go to 9999
8000  continue
      write(*,'(a)')'input1d: EOF encountered when searching for
     & requested start time.'
      nent = -1
9999  continue
      nent = -1
      return
      end
