c----------------------------------------------------------------------
      subroutine rdhead(ind,infile,imark,*)

	implicit none



	include 'ctrlblk.inc'
c----------------------------------------------------------------------
c  Reads the window and header information from the data file produced
c  by the RED2BIN program.  Ron Kurtz, June 7, 1988.
c  July 1, 1990:
c       Modified to read binary files
c  November 8,1990:
c       Modified to remove both hardware and software gains.
c
c  August 19, 1992 (Angela Tchen)
c       Separated from parent program file lims_cascade.f
c       logical variable SUN was added to common block /contst/
c       modified for use on the SUN
c
c  Sept 8, 1993  (D. Boerner)
c       added common block to get at header info. 
c
c  Sept 30, 1993 (J. Craven)
c       soft coded SUN/PC dependence to improve portability
c
c Changed 'UNFORMATTED' to 'BINARY' for MS Fortran
C  Feb  4, 1994   J.Craven 
c***********************************************************************
c Variables Declared:
c
c INTEGERS
c byy    - start time year
c bmm    - start time month
c bdd    - start time day
c bhh    - start time hour
c bmin   - start time minute
c bss    - start time second
c byyin  - input byy; if set to 9999 then question re block start time
c                     not asked
c itgain - hardware telluric gain
c lenex  - Ex line length (metres)
c leney  - Ey line length (metres)
c lbox   - serial number of the lunchbox
c trkchn - total number of active channels (NOT temperature or voltage)
c nchan  - total number of data channels recorded (does not include the
c 	   auxiliary channels) (a.k.a. dchan in lims_cascade program)
c ind    - a read unit
c prterr - to indicate level of error for user
c j      - a counter for a number of variables
c gain(16)   - the gain setting for different channels 
c baseln(3)  - the baselines for measurement.
c mapcmp(16) - the map for the auxiliary channels
c
c INTEGER*2
c intgr(29) - different variables read in from window (see comments below)
c filtp1    - filter type (analog or digital) in GROUP1
c filtp2    - filter type (analog or digital) in GROUP2
c chnum     - channel number
c gain2     - gain information read in
c
c REAL
c smprt1 - sample rate in GROUP1
c smprt2 - sample rate in GROUP2
c delta  - digitizing interval in Hz.
c magfac - magnetometer factor.
c
c CHARACTER
c comp(16)*2- contains the channel name (ie. Hx, Ey, etc.)  Its subscript
c	corresponds to the actual LIMS channel number.
c prjcod*3  - project number
c runnum*7  - run number
c sitdes*20 - site description
c latdir*1  - latitude direction
c lngdir*1  - longitude direction
c azdir*17  - azimuth direction
c smpin1*4  - sample interval (Hz or seconds) in GROUP1
c smpin2*4  - sample interval (Hz or seconds) in GROUP2
c compnt*4  - channel information read in
c
c************************************************************************
c
      integer byy, bmm, bdd, bhh, bmin, bss, stsec, byyin
      integer itgain, lenex, leney, trkchn, idel, f10
      integer nchan, ind, prterr, lbox, imark
      integer gain(16), baseln(3), mapcmp(16)

      integer*2 intgr(29), filtp1, filtp2, chnum, gain2

      real smprt1, smprt2
      real delta, magfac

      character cfile*7, comp(16)*2, infile*(*)
      character prjcod*3, runnum*7, sitdes*20, latdir*1, lngdir*1
      character azdir*17, smpin1*4, smpin2*4, compnt(16)*4 

      LOGICAL ANS

      integer*4 inc, j, lf

      common /contst/ inc, delta, prterr, nchan
      common /blkst/  byy, bmm, bdd, bhh, bmin, bss
      common /calbtn/ itgain, lenex, leney, magfac, gain, lbox, comp,
     >                mapcmp, cfile
      common /header/ prjcod,runnum,sitdes,latdir,lngdir,azdir,
     >                smpin1,smpin2
      common /headr2/ smprt1, smprt2, baseln, intgr, filtp1, filtp2

c************************************************************************
c  Read in the window information.
c    VARIABLE                              COUNT      TYPE  LENGTH   INTGR
c                                                           (bytes)
c    project code                              3      char       3
c    run number                                7      char       7
c    window number                             1      int        2       1
c    site description                         20      char      20
c    Ex line length (metres)                   1      int        2       2
c    Ey line length (metres)                   1      int        2       3
c    latitude (degrees)                        1      int        2       4
c    latitude (minutes)                        1      int        2       5
c    latitude (seconds)                        1      int        2       6
c    latitude direction (N or S)               1      char       1
c    longitude (degrees)                       1      int        2       7
c    longitude (minutes)                       1      int        2       8
c    longitude (seconds)                       1      int        2       9
c    longitude direction (E or W)              1      char       1
c    hardware telluric gain                    1      int        2      10
c    telluric filter settling time (min)       1      int        2      11
c    Ex azimuth (degrees)                      1      int        2      12
c    Ey azimuth (degrees)                      1      int        2      13
c    Hx azimuth (degrees)                      1      int        2      14
c    az direction (from true or magntc north) 17      char      17
c    serial number of lunchbox                 1      int        2      15
c    serial number of magnetometer             1      int        2      16
c    serial number of tellurics                1      int        2      17
c    start time DAY                            1      int        2      18
c    start time MONTH                          1      int        2      19
c    start time YEAR                           1      int        2      20
c    start time HOUR                           1      int        2      21
c    start time MINUTE                         1      int        2      22
c    duration DAYS                             1      int        2      23
c    duration HOURS                            1      int        2      24
c    duration MINUTES                          1      int        2      25
c    magnetometer factor=25600 microvolts/bit  1      int        2      26
c    A to D factor = 305 microvolts/bit        1      int        2      27
c    number of channels in GROUP1              1      int        2      28
c    number of channels in GROUP2              1      int        2      29
c*************************************************************************

      byyin = byy
c
c LIMS magnetomter values are given in microVolts. To convert to nT, remove 
c the software gain and divide by 10000. ( 10 milliVolts = 1 nT )
c
      magfac = 10000.

#ifdef SUN
c      open(ind,file=infile,status='old',form='formatted',err=7000)
c      ind = getfilep(ind)

      if( iprint.ge.1 ) then

	  write(*,*)'rdhead: ifdef SUN used'

	endif


      call mpopen(ind,infile,'old',*7100,*7000)
c
      call readst(ind,prjcod,3,*8000,*8000)
      call readst(ind,runnum,7,*8000,*8000)
      call readi2(ind,intgr(1),*8000,*8000)
      call readst(ind,sitdes,20,*8000,*8000)
      do j = 2, 6
         call readi2(ind,intgr(j),*8000,*8000)
      end do
      call readst(ind,latdir,1,*8000,*8000)
      call readi2(ind,intgr(7),*8000,*8000)
      call readi2(ind,intgr(8),*8000,*8000)
      call readi2(ind,intgr(9),*8000,*8000)
      call readst(ind,lngdir,1,*8000,*8000)
      do j = 10, 14
         call readi2(ind,intgr(j),*8000,*8000)
      end do
      call readst(ind,azdir,17,*8000,*8000)
      do j = 15, 29
         call readi2(ind,intgr(j),*8000,*8000)
      end do
#elif ASCII
      if( iprint.ge.1 ) then

	  write(*,*)'rdhead: ifdef ASCII used'

	endif

      open(ind,file=infile,status='old',err=7100)
      read( ind, 5000 ) prjcod,runnum,intgr(1),sitdes,(intgr(j),j=2,6)
      read( ind, 5001 ) latdir, intgr(7), intgr(8), intgr(9), lngdir
      read( ind, 5002 ) (intgr(j), j=10,14), azdir,(intgr(j),j=15,29)
5000  format( a3, a7, i4, a20, 5i4 )
5001  format( a1, 3i4, a1 )
5002  format( 5i4, a17, 11i4, i6, 3i4 )
#elif DOS
C$$$      open(ind,file=infile,status='old',form='unformatted',err=7100)
      if( iprint.ge.3 ) then

	  write(*,*)'rdhead: ifdef DOS used'

	  call pause( 'Continue' )

	endif

      open(ind,file=infile,status='old',form='binary',err=7100)

	if( iprint.ge.3 ) then

	  write(*,*)'rdhead: file opened: ', infile

	  call pause( 'Continue' )

	endif
      read( ind )prjcod, runnum, intgr(1), sitdes, (intgr(j), j=2,6)

	if( iprint.ge.3 ) then

	  write(*,*)'rdhead: first line read in from file: prjcod=',

     &            prjcod

	  call pause( 'Continue' )

	endif


      read( ind )latdir, intgr(7), intgr(8), intgr(9), lngdir
	if( iprint.ge.3 ) then

	  write(*,*)'rdhead: second line read in from file: latdir=',

     &            latdir

	  call pause( 'Continue' )

	endif



      read( ind )(intgr(j), j=10,14), azdir, (intgr(j), j=15,29)

      if( iprint.ge.3 ) then

	  write(*,*)'rdhead: third line read in from file: azdir=',

     &            azdir

	  call pause( 'Continue' )

	endif


#endif

      if( prterr .ge. 2 )then
        write(*,*) ' Dump of header information from rdhedd:'
        write(*,*) prjcod, runnum, intgr(1), sitdes, (intgr(j),j=2,6)
        write(*,*) latdir, intgr(7), intgr(8), intgr(9), lngdir
        write(*,*) (intgr(j), j=10,14), azdir, (intgr(j), j=15,29)
	  call pause( 'Continue' )

      endif

c  Obtain the year.
      byy = intgr(20)

c  Set the instrument telluric gain.
      itgain = intgr(10)

c  Set the telluric line length.
      lenex = intgr(2)
      leney = intgr(3)

c  Set the lunchbox serial number
      lbox  = intgr(15)

      do j = 1, 16
        comp(j) = '  '
        gain(j) = 0
      enddo
      comp(11) = 'A1'
      comp(12) = 'A2'
      comp(13) = 'A3'
      comp(14) = 'Ti'
      comp(15) = 'Th'
      comp(16) = 'Bt'
c
c     Set bss equal to zero as data blocks always start on the minute.
      bss = 0

c     Set the start time of the data block
      bmm  = intgr(19)
      bdd  = intgr(18)
      bhh  = intgr(21)
      bmin = intgr(22)
c
      if( prterr .gt. 0 ) then
        write(*,'(a,a,i4,5i3)')'Data start time: year/month/day/',
     &       'hour/min/sec=', byy, bmm, bdd, bhh, bmin, bss

	  call pause( 'Continue' )
      endif

      if( byyin.ne.9999 ) then
        ANS = .FALSE.
        call lin(' Do you wish to change the datablock start time?',ANS)
        if ( ANS ) then
          call consec (stsec, byy, bmm, bdd, bhh, bmin, bss )
          call iin( ' Give new block data start time ', stsec )
          call seccon (stsec, byy, bmm, bdd, bhh, bmin, bss )
          write(*,'(a,a,i4,5i3)')'Data start time: year/month/day',
     &                '/hour/min/sec=', byy, bmm, bdd, bhh, bmin, bss
        end if
      end if

c     trkchn: total number of active channels (NOT temperature or voltage)
      trkchn = 0

c  The subscript of mapcmp corresponds to the plotting channel number and
c     to the entries stored in the data output file.
c     The value of mapcmp is the actual LIMS channel number.

c Read in any GROUP1 channel information.
c filtp1:   filter type (analog or digital)
c smprt1:   sample rate
c smpin1:   sample interval (hz or seconds)
c comp(16): contains the channel name (ie. Hx, Ey etc.).  Its subscript
c           corresponds to the actual LIMS channel number.

      if ( intgr(28) .gt. 0 ) then
#ifdef SUN
         call readi2(ind,filtp1,  *8000,*8000)
         call readfl(ind,smprt1,  *8000,*8000)
         call readst(ind,smpin1,4,*8000,*8000)
#elif ASCII
         read (ind,*) filtp1, smprt1, smpin1
#elif DOS
         read (ind ) filtp1, smprt1, smpin1
#endif

        do 200 j = 1, intgr(28)
#ifdef SUN
          call readi2(ind,chnum,      *8000,*8000)
          call readst(ind,compnt(j),4,*8000,*8000)
          call readi2(ind,gain2,      *8000,*8000)
#elif ASCII
          read ( ind,'(i2,a2,2x,i4)' ) chnum, compnt(j), gain2
#elif DOS
          read ( ind ) chnum, compnt(j), gain2
#endif
          if( prterr.ge.3 ) then
              write(*,*)'Channel number, component &  gain =>',
     &                  chnum, compnt(j), gain2
          endif
          comp(chnum) = compnt(j)
          mapcmp(j) = chnum
          gain(chnum) = gain2
          trkchn = trkchn + 1
  200   continue          
      end if

c  Read in any GROUP2 channel information.
      if ( intgr(29) .gt. 0 ) then
#ifdef SUN
        call readi2(ind,filtp2,  *8000,*8000)
        call readfl(ind,smprt2,  *8000,*8000)
        call readst(ind,smpin2,4,*8000,*8000)
#elif ASCII
        read ( ind,* ) filtp2, smprt2, smpin2
#elif DOS
        read ( ind ) filtp2, smprt2, smpin2
#endif

        do j = 1, intgr(29)
          trkchn = trkchn + 1
#ifdef SUN
          call readi2(ind,chnum,*8000,*8000)
          call readst(ind,compnt(trkchn),4,*8000,*8000)
          call readi2(ind,gain2,*8000,*8000)
#elif ASCII
          read ( ind,* ) chnum, compnt(trkchn), gain2
#elif DOS
          read ( ind ) chnum, compnt(trkchn), gain2
#endif
          comp(chnum) = compnt(trkchn)
          mapcmp(trkchn) = chnum
          gain(chnum) = gain2
        end do
      end if
c
c     nchan: total number of data channels recorded (does not include 
c            temperature and voltage channels).
c
      nchan = trkchn
c
c  Define the map for the auxiliary channels.
c
      do j = 1, 6
         mapcmp(nchan+j) = 10 + j
      end do
c
c  Calculate the digitizing interval in Hz.
c
      if ((smpin1(1:1) .eq. 'h') .or. (smpin1(1:1) .eq. 'H')) then
        delta = smprt1
      else
        delta = 1.0/smprt1
      end if
c
c Calculate the number of samples before the first timing mark
c where the auxilliary channels are stored.
c
      f10 = 10 - MOD(bmin, 10)
      if ( delta .gt. 1.0 ) then
        imark = f10 * 60 * ifix (delta) + 1
       else
        idel =  nint (1.0/delta)
        imark = (f10 * 60/idel) + 1
      end if
c
c  Read in the magnetometer baseline values in LIMS units.
c     To convert to Microvolts multiply by 25600.

#ifdef SUN
      do j = 1, 3
         call readi4(ind,baseln(j),*8000,*8000)
      end do
#elif ASCII
      read (ind,*,err=8000,end=8000) (baseln (j), j= 1, 3)
#elif DOS
      read (ind,err=8000,end=8000) (baseln (j), j= 1, 3)
#endif

      if ( prterr .ge. 2 ) then
        write (*, '(2x, a)') 'Dump of selected header information:'
        write (*, '(1x, a, i5)')   'byy    = ', byy
        write (*, '(1x, a, i5)')   'bmm    = ', bmm
        write (*, '(1x, a, i5)')   'bdd    = ', bdd
        write (*, '(1x, a, i5)')   'bhh    = ', bhh
        write (*, '(1x, a, i5)')   'bmin   = ', bmin
        write (*, '(1x, a, i5)')   'itgain = ', itgain
        write (*, '(1x, a, i5)')   'lenex  = ', lenex
        write (*, '(1x, a, i5)')   'leney  = ', leney
        write (*, '(1x, a, f8.3)') 'delta  = ', delta
        write (*, '(1x, a, 3i10)') 'baselines = ', (baseln(j), j=1,3)
        write (*, '(1x, a, i5)') 'nchan = ', nchan
        write (*,'(a)')' COMP  MAPCMP  GAIN'
        do j = 1, 16
          write (*, '(a4, i7, i6)') comp(j), mapcmp(j), gain(j)
        end do
      end if
c
      return
7000  write(*,*) 'rdhead.F New file could not be opened ', infile
      write(*,*)' How did you get to this piece of code?'
      stop
7100  call trunc(infile,lf)
      write(*,*)'rdhead.F: Error reading old file ', infile(1:lf),
     $     ' File likely does NOT EXIST'
      return 1
8000  write(*,*) 'End of File or Error found during read!'
      return
      end
