      subroutine jknvar( NEW, ntot, s, iflag, zjack, d1diff, var )
      implicit none


      include 'maxspc.inc'
      include 'ctrlblk.inc'


      logical NEW
      complex s(7,7,*), zjack(4)
      complex z(MAXSPC,4), zd1(MAXSPC,4), zall(4), zd1sum(4)
      complex csum, czero
      real    var(4), d1diff(MAXSPC,4), xn, factor
      integer iflag(*), ntot, n, isum, i, j
      character type(4)*3/'ZXX','ZXY','ZYX','ZYY'/
      external csum



c     NEW      logical: if TRUE  then assumed tf's need to be calculated
c                          FASLE                   already exist
c
c    ntot      integer:      total number of estimates available
c    s         compex array: ntot estimates of cross powers 
c    iflag     int array:    set to 1 if used, 0 to reject

      czero = cmplx(0.,0.)
c
      if( ntot.gt.MAXSPC ) then
        write(*,*)'jknvar: ntot>MAXSPC:, ntot=',ntot,'  MAXSPC=',
     &            MAXSPC
        stop
      end if

      n = isum( iflag, ntot )
c
      if( iprint.ge.2 ) then
        write(*,*)'jknvar: ntot, n = ', ntot, ',', n
        write(*,'(a,51i1,10(/,70i1))')'         iflag   = ', 
     &                                (iflag(i),i=1,n)
      end if
c
      if(n.le.1) then
        var(1)=1.0e+30
        var(2)=1.0e+30
        var(3)=1.0e+30
        var(4)=1.0e+30
        return
      end if
c
      xn = float( n )
      factor = (xn-1.)/xn

c---compute individual estimates of transfer functions
c   only do this for NEW=.TRUE.

      if( NEW ) then
        do i = 1, ntot
          if( iflag(i).eq.1 ) then
            call trfn( 7, s(1,1,i), 1, 2, 6, 7, 3, z(i,1), z(i,2) )
            call trfn( 7, s(1,1,i), 1, 2, 6, 7, 4, z(i,3), z(i,4) )
          else
            do j = 1, 4
              z(i,j) = czero
            enddo
          endif
        enddo
      else
        do i = 1, ntot
          if( iflag(i).eq.0 ) then
            do j = 1, 4
              z(i,j) = czero
            enddo
          endif
        enddo
      endif


c---construct estimates using all available
      do j = 1, 4
        zall(j) = csum(z(1,j),ntot)/xn
      end do

      if( iprint.ge.2 ) then
        write(*,*)'Z-all estimates ==>'
        do j = 1, 4
          write(*,*) type(j), zall(j)
        end do
      end if
c
c---construct delete-1 estimates
c
      do i = 1, ntot
        if( iflag(i).eq.1 ) then
          do j = 1, 4
            zd1(i,j) = (xn*zall(j) - z(i,j))/(xn-1.)
          end do
        else
          do j = 1, 4
            zd1(i,j) = czero
          end do
        end if
      end do
c
c---construct jacknife estimates of "Z"
c
      do j = 1, 4
        zjack(j) = xn*zall(j) - factor*csum(zd1(1,j),ntot)
      end do

      if( iprint.ge.2 ) then
        write(*,*)'Z jacknife estimates ==>'
        do j = 1, 4
          write(*,*) type(j), zjack(j)
        end do
      end if
c
c---sum delete-1 estimates
c
      do j = 1, 4
        zd1sum(j) = csum( zd1(1,j), ntot )
      end do
c
c---and normalize by 'N'
c
      do j = 1, 4
        zd1sum(j) = zd1sum(j)/xn
      end do

      if( iprint.ge.2 ) then
        write(*,*)'Delete-1 sums  ==>'
        do j = 1, 4
          write(*,*) type(j), zd1sum(j)
        end do
      end if
c
c---construct variances
c
      do j = 1, 4
        var(j) = 0.0
      end do

      do i = 1, ntot
        if( iflag(i).eq.1 ) then
          do j = 1, 4
            d1diff(i,j) = cabs(zd1(i,j) - zd1sum(j))**2.
            var(j) = var(j) + d1diff(i,j)
          end do
        end if
      end do

      if( iprint.ge.3 ) then
        write(*,*)'Delete-1 differences'
        do i = 1, ntot
          if( iflag(i).eq.1 ) then
            write(*,*) i, (d1diff(i,j),j=1,4)
          end if
        end do
      end if

      do j = 1, 4
        var(j) = factor * var(j)
      end do

      if( iprint.ge.2 ) then
        write(*,*)'Variances  ==>'
        do j = 1, 4
          write(*,*) type(j), var(j)
        end do
      end if
c
      return
      end

c-------------------------------------------------------------------------------
      subroutine jknvarmin( ntype, ntot, s, iflag, varl )

	implicit none

c---modification history
c
c   agj: 21/09/1994: added "ntype" to minimize a selected element
c
c
      include 'maxspc.inc'
      include 'ctrlblk.inc'


      complex s(7,7,*), zjack(4)
      complex csum
      real var0(4), var(4), varl(4), d1diff(MAXSPC,4), d1diffmax(4)
      real vari, var1, var1l
      integer iflag(*), maxd(4), ntot, i, j, n, icycle, isum
      integer kxy, kyx, nrej, ntype
      character type(4)*3/'ZXX','ZXY','ZYX','ZYY'/
      external csum


c
c     ntype = 1: Minimize sqrt(Var(ZXY) * Var(ZYX))
c     ntype = 2: Minimize      Var(ZXY)
c     ntype = 3: Minimize                 Var(ZYX)

      if( iprint.ge.1 ) then
        write(*,*)'Entered jknvarmin: ntype, ntot = ', ntype, ntot
      endif


c---compute initial values of variances

      do i = 1, ntot
        iflag(i) = 1
      end do

      call jknvar( .TRUE., ntot, s, iflag, zjack, d1diff, var0 )

      if( ntype.eq.1 ) then
        vari = sqrt(var0(2)*var0(3))
      else if( ntype.eq.2 ) then
        vari = var0(2)
      else if( ntype.eq.3 ) then
        vari = var0(3)
      else
        write(*,*)'Unacceptable ntype = ', ntype
        stop
      endif

      if( iprint.ge.1 ) then
        write(*,*)'Initial variance =>', vari
      endif

      if( iprint.ge.2 ) then
        write(*,*)'Initial impedances =>'
        do j = 1, 4
          write(*,*) type(j), zjack(j)
        end do
      end if

      if( ntype.eq.1 ) then
        var1l = sqrt(var0(2)*var0(3))
      else if( ntype.eq.2 ) then
        var1l = var0(2)
      else if( ntype.eq.3 ) then
        var1l = var0(3)
      endif

      n = ntot

      do icycle = 1, ntot-1

c---find largest contribution to d1diff
  
        do j = 1, 4
          d1diffmax(j) = -1.
        end do
        do i = 1, ntot
          if( iflag(i).eq.1 ) then
            do j = 1, 4
              if( d1diff(i,j).gt.d1diffmax(j) ) then
                maxd(j) = i
                d1diffmax(j) = d1diff(i,j)
              end if
            end do
          end if
        end do

        kxy = maxd(2)
        kyx = maxd(3)

        if( iprint.ge.2 ) then
          write(*,*)'Cycle = ', icycle, ' delete-1 diffs maxima'
          write(*,*) (maxd(j),':', d1diffmax(j), j=1,4)
        end if


c---compute variances without kxy'th & kyx'th estimates
        if( ntype.eq.1 ) then
          iflag(kxy) = 0
          iflag(kyx) = 0
        else if( ntype.eq.2 ) then
          iflag(kxy) = 0
        else if( ntype.eq.3 ) then
          iflag(kyx) = 0
        endif
        n = isum( iflag, ntot )
        call jknvar( .FALSE., ntot, s, iflag, zjack, d1diff, var )

        if( ntype.eq.1 ) then
          var1 = sqrt(var(2)*var(3))
        else if( ntype.eq.2 ) then
          var1 = var(2)
        else if( ntype.eq.3 ) then
          var1 = var(3)
        endif

        if( iprint.ge.3 ) then
          write(*,*)'Cycle,n =',icycle,n,' variance =>', var1
        end if

        if( var1l .gt. var1 ) then
          do j = 1, 4
            varl(j) = var(j)
          end do
          if( ntype.eq.1 ) then
            var1l = sqrt(var(2)*var(3))
          else if( ntype.eq.2 ) then
            var1l = var(2)
          else if( ntype.eq.3 ) then
            var1l = var(3)
          endif
        else
          if( ntype.eq.1 ) then
            iflag(kxy) = 1
            iflag(kyx) = 1
          else if( ntype.eq.2 ) then
            iflag(kxy) = 1
          else if( ntype.eq.3 ) then
            iflag(kyx) = 1
          endif
          n = isum( iflag, ntot )
          nrej = ntot - n
          goto 100
        end if

        if( n.le.3 ) goto 110

      end do

      if( iprint.ge.1 ) then
        write(*,*)'Done all cycles!!'
      endif
      return

100   if( iprint.ge.1 ) then
        write(*,*)'Variances > Last; nrej = ', nrej, ' variance=>',
     &          var1l
      endif
      if( iprint.ge.2 ) then
        call jknvar( .TRUE., ntot, s, iflag, zjack, d1diff, var0 )
        write(*,*)'Final impedances =>'
        do j = 1, 4
          write(*,*) type(j), zjack(j)
        end do
      endif
      return

110   if( iprint.ge.1 ) then
        write(*,*)'Cannot reject any more!!'
      endif
      if( iprint.ge.2 ) then
        call jknvar( .TRUE., ntot, s, iflag, zjack, d1diff, var0 )
        write(*,*)'Final impedances =>'
        do j = 1, 4
          write(*,*) type(j), zjack(j)
        end do
      endif
      return

      end

c-------------------------------------------------------------------------------
      subroutine jknvarmin1( ntot, s, iflag, varl )

	implicit none

      include 'maxspc.inc'
      include 'ctrlblk.inc'


      complex s(7,7,*), zjack
      complex csum
      real var0, var, varl, d1diff(MAXSPC), d1diffmax
      integer iflag(*), maxd, ntot, i, k, icycle, isum, nrej, n
      character type(4)*3/'ZXX','ZXY','ZYX','ZYY'/
      external csum

      write(*,*)'Entered jknvarmin1: ntot = ', ntot


c---compute initial values of variances

      do i = 1, ntot
        iflag(i) = 1
      end do

      call jknvar1( .TRUE., ntot, s, iflag, zjack, d1diff, var0 )

      write(*,*)'Initial mean variance =>', var0

      if( iprint.ge.2 ) then
        write(*,*)'Initial impedances =>',  zjack
      end if

      varl = var0

      n = ntot

      do icycle = 1, ntot-1

c---find largest contribution to d1diff
  
        d1diffmax = -1.

        do i = 1, ntot
          if( iflag(i).eq.1 ) then
            if( d1diff(i).gt.d1diffmax ) then
              maxd = i
              d1diffmax = d1diff(i)
            endif
          endif
        enddo

        k = maxd

        if( iprint.ge.2 ) then
          write(*,*)'Cycle = ', icycle, ' delete-1 diff maximum',
     &               maxd, d1diffmax
        end if


c---compute variances without k'th estimate
        iflag(k) = 0
        n = isum( iflag, ntot )
        call jknvar1( .FALSE., ntot, s, iflag, zjack, d1diff, var )

        if( iprint.ge.2 ) then
          write(*,*)'Cycle,n =',icycle,n,'  variance =>',var
        end if

        if( varl .gt. var ) then
          varl = var
        else
          iflag( k ) = 1
          n = isum( iflag, ntot )
          nrej = ntot - n
          goto 100
        end if

        if( n.le.3 ) goto 110

      end do

      write(*,*)'Done all cycles!!'
      return

100   write(*,*)'Variances > Last; nrej = ', nrej, ' mean variance=>',
     &          var

      return

110   write(*,*)'Cannot reject any more!!'
      return

      end

c-------------------------------------------------------------------------------
      subroutine jknvar1( NEW, ntot, s, iflag, zjack, d1diff, var )

	implicit none

      include 'maxspc.inc'
      include 'ctrlblk.inc'

      logical NEW
      complex s(7,7,*), zjack
      complex z(MAXSPC), zd1(MAXSPC), zall, zd1sum
      complex csum, czero, z1, z2
      real var, d1diff(MAXSPC), xn, factor
      integer iflag(*), ntot, n, isum, i
      character type(4)*3/'ZXX','ZXY','ZYX','ZYY'/
      external csum



c     NEW      logical: if TRUE  then assumed tf's need to be calculated
c                          FALSE                   already exist
c
c    ntot      integer:      total number of estimates available
c    s         compex array: ntot estimates of cross powers 
c    iflag     int array:    set to 1 if used, 0 to reject

      czero = cmplx(0.,0.)


      if( ntot.gt.MAXSPC ) then
        write(*,*)'jknvar1: ntot>MAXSPC:, ntot=',ntot,'  MAXSPC=',
     &            MAXSPC
        stop
      end if

      n = isum( iflag, ntot )
      if(n.le.1) then
        var=1.0e+30
        return
      end if
      xn = float( n )
      factor = (xn-1.)/xn

      if( iprint.ge.2 ) then
        write(*,*)'jknvar1: ntot, n = ', ntot, ',', n
        write(*,'(a,51i1,10(/,70i1))')'         iflag   = ', 
     &                                (iflag(i),i=1,n)
      end if

c---compute individual estimates of transfer functions
c   only do this for NEW=.TRUE.

      if( NEW ) then
        do i = 1, ntot
          if( iflag(i).eq.1 ) then
            call trfn1( 7, s(1,1,i), 1, 6, 5, z1 )
            call trfn1( 7, s(1,1,i), 1, 7, 5, z2 )
            z(i) = csqrt(z1*z2)
          else
            z(i) = czero
          endif
        enddo

      else
        do i = 1, ntot
          if( iflag(i).eq.0 ) then
            z(i) = czero
          endif
        enddo

      endif


c---construct estimates using all available
      zall = csum(z,ntot)/xn

      if( iprint.ge.2 ) then
        write(*,*)'Z-all estimate ==>', zall
      end if


c---construct delete-1 estimates of z-all
      do i = 1, ntot
        if( iflag(i).eq.1 ) then
          zd1(i) = (xn*zall - z(i))/(xn-1.)
        else
          zd1(i) = czero
        end if
      end do


c---construct jacknife estimates of "Z"
      zjack = xn*zall - factor*csum(zd1,ntot)

      if( iprint.ge.2 ) then
        write(*,*)'Z jacknife estimate ==>', zjack
      end if


c---sum delete-1 estimate
      zd1sum = csum( zd1, ntot )

c---and normalize by 'N'
      zd1sum = zd1sum/xn

      if( iprint.ge.2 ) then
        write(*,*)'Delete-1 sum  ==>', zd1sum
      end if


c---construct variance
      var = 0.0

      do i = 1, ntot
        if( iflag(i).eq.1 ) then
          d1diff(i) = cabs(zd1(i) - zd1sum)**2.
          var       = var + d1diff(i)
        end if
      end do

      if( iprint.ge.3 ) then
        write(*,*)'Delete-1 differences =>'
        do i = 1, ntot
          if( iflag(i).eq.1 ) then
            write(*,*) i, d1diff(i)
          end if
        end do
      end if

      var = factor * var
      if( iprint.ge.2 ) then
        write(*,*)'      variance  ==>', var
      end if
      return
      end

