c======================================================================
      subroutine stack7( mt, m, iv, y, z )
c
c     subroutine to include mini-stacks into the main stack after
c     deriving the weighting function
c
c     i/p
c           mt: character*1 variable
c                            = M: MT  processing
c                            = G: GDS processing
c                            = H: HSG processing
c           m           the band to be stacked
c           iv(*)       the number of records in each stack
c           y(*,*)      6'th harmonic cross-spectra
c           z(*,*)      8'th harmonic cross-spectra
c
c    o/p
c           no formal output except than iv(m), y(m,*) & z(m,*)
c           reset to zero on exit
c
c    for m = -1 then iv, y, & z are returned with "tot"
c

 

      include 'maxflims.inc'
      include 'io.inc'

      include 'ctrlblk.inc'


      integer ntot, prterr, nz, ny, dchan
      parameter ( NTOT = 49*MAXF2 )

      real y(MAXF2,49), z(MAXF2,49), ytot(MAXF2,49), ztot(MAXF2,49),
     &     wytot(MAXF2), wztot(MAXF2), ytemp(49), ztemp(49), f(0:64),
     &     ochdy, ochdz, mcnhdy, mcnhdz, mcehdy, mcehdz,
     &     mczhdy, mczhdz
      real delta, wez, whz, wdz, wz, wy, why, frccf2, fmcoh
      real wey, wnz, oczhy, oczhz, zz, yy, focoh, wny, wdy, frccf1
      complex sy(7,7), sz(7,7)
      integer iv(MAXF2), ivtot(MAXF2), m, nfw(MAXF)
      integer inc, i, j
      logical JJANAL
      character*1 mt
 
      external focoh, fmcoh
 
      save ytot, ztot, wytot, wztot, ivtot, ytemp, ztemp
 
      common /contst/ inc, delta, prterr, dchan
      common /yz/  yy(64,49,MAXF2), zz(64,49,MAXF2), JJANAL
      common /degf/ nfw
 
      data ytot/NTOT*0./, ztot/NTOT*0./, 
     &     wytot/MAXF2*0./, wztot/MAXF2*0./, ivtot/MAXF2*0/

 
c---F values for F(4:n-4:0.05); where n = 2 x no. of estimates
c   F(0) deals with special case where ny or nz = 0
c
       data f/0.0,
     &        0.0,  0.0,19.25, 6.39, 4.53, 3.84, 3.48, 3.25,
     &        3.11, 3.01, 2.93, 2.87, 2.82, 2.78, 2.74, 2.71,
     &        2.69, 2.67, 2.65, 2.62, 2.61, 2.60, 2.59, 2.58,
     &       2.574,2.565,2.557,2.550,2.543,2.537,2.531,2.525,
     &       2.520,2.515,2.511,2.507,2.503,2.499,2.495,2.492,
     &       2.489,2.486,2.483,2.480,2.478,2.475,2.473,2.471,
     &       2.469,2.466,2.465,2.463,2.461,2.459,2.457,2.456,
     &       2.454,2.453,2.451,2.450,2.449,2.447,2.446,2.445 /
 
      if( m.lt.0 ) goto 20
 
      if( iprint.ge.2 ) then
        write(*,*) 'stack7: entered for band =', m
      endif
 
      if( JJANAL ) then
        call minvar7( mt, iv(m), yy(1,1,m), ny, ytemp )
        call minvar7( mt, iv(m), zz(1,1,m), nz, ztemp )
        if( iprint.ge.2 ) write(*,'(a,i2,a,i2)')' Accepted = ',
     &                                       ny, '/', nz
        if( ny.eq.0 .and. nz.eq.0 ) then
          if( iprint.ge.1 ) write(*,*)'stack7: left with ny=nz=0'
          iv(m) = 0
          return
        endif
      else
        ny = iv(m)
        nz = iv(m)
        do i = 1, 49
          ytemp(i) = y(m,i)
          ztemp(i) = z(m,i)
        enddo
      end if
 
      nfw(2*m  ) = nfw(2*m  ) + ny
      nfw(2*m-1) = nfw(2*m-1) + nz
 
      write(outd2) m, ny, (ytemp(i),i=1,49), nz, (ztemp(i),i=1,49)
 
 
c---write auto & cross-spectra into S arrays
      call acspec7( ytemp, sy )
      call acspec7( ztemp, sz )
 
c---derive weighting function for mini-stack held in y(*,*) & z(*,*)
c   weighting is function of variance given by
c
c   var(xy)  =   4        (1-mcoh(ex,hx,hy))   S(ex)
c              ------- F  ------------------  -------
c              n - 4      (1-ocoh(hx,hy)   )   S(hy)
c
c
 
c---ocoh(hx,hy)
      ochdy = focoh( 7, sy, 1, 2 )
      ochdz = focoh( 7, sz, 1, 2 )
      ochdy = min( 0.999, ochdy )
      ochdz = min( 0.999, ochdz )

      if( mt.eq.'M' ) then 
c---mcoh(ex,hx,hy)
        mcnhdy = fmcoh( 7, sy, 3, 1, 2 )
        mcnhdz = fmcoh( 7, sz, 3, 1, 2 )
        mcnhdy = min( 0.999, mcnhdy )
        mcnhdz = min( 0.999, mcnhdz )
 
c---mcoh(ey,hx,hy)
        mcehdy = fmcoh( 7, sy, 4, 1, 2 )
        mcehdz = fmcoh( 7, sz, 4, 1, 2 )
        mcehdy = min( 0.999, mcehdy )
        mcehdz = min( 0.999, mcehdz )

      else if( mt.eq.'G' ) then
c---mcoh(hz,hx,hy)
        mczhdy = fmcoh( 7, sy, 5, 1, 2 )
        mczhdz = fmcoh( 7, sz, 5, 1, 2 )
        mczhdy = min( 0.999, mczhdy )
        mczhdz = min( 0.999, mczhdz )

      else if( mt.eq.'H' ) then
c---ocoh(hz,hx)
        oczhy = focoh( 7, sy, 5, 1 )
        oczhz = focoh( 7, sz, 5, 1 )
        oczhy = min( 0.999, oczhy )
        oczhz = min( 0.999, oczhz )

      end if

c---weights
 
      if( mt.eq.'M' ) then
        wny = frccf2( 2*ny, f(ny), mcnhdy, ochdy,
     &                      real(sy(3,3)), real(sy(2,2)) )
        wey = frccf2( 2*ny, f(ny), mcehdy, ochdy,
     &                      real(sy(4,4)), real(sy(1,1)) )
        wnz = frccf2( 2*nz, f(nz), mcnhdz, ochdz,
     &                      real(sz(3,3)), real(sz(2,2)) )
        wez = frccf2( 2*nz, f(nz), mcehdz, ochdz,
     &                      real(sz(4,4)), real(sz(1,1)) ) 
        wy = 1./sqrt(wny*wey)
        wz = 1./sqrt(wnz*wez)

      else if( mt.eq.'G' ) then
        why = frccf2( 2*ny, f(ny), mczhdy, ochdy,
     &                      real(sy(5,5)), real(sy(1,1)) )
        wdy = frccf2( 2*ny, f(ny), mczhdy, ochdy,
     &                      real(sy(5,5)), real(sy(2,2)) )
        whz = frccf2( 2*nz, f(nz), mczhdz, ochdz,
     &                      real(sz(5,5)), real(sz(1,1)) )
        wdz = frccf2( 2*nz, f(nz), mczhdz, ochdz,
     &                      real(sz(5,5)), real(sz(2,2)) )
        wy = 1./sqrt(why*wdy)
        wz = 1./sqrt(whz*wdz)

      else if( mt.eq.'H' ) then
        wy = frccf1( 2*ny, f(ny), oczhy, real(sy(5,5)), real(sy(1,1)) )
        wz = frccf1( 2*nz, f(nz), oczhz, real(sz(5,5)), real(sz(1,1)) )

      endif 

      if( iprint.ge.2 ) write(*,*)' weights ==>', wy, wz
 
      ivtot(m) = ivtot(m) + iv(m)
      iv(m) = 0
      wytot(m) = wytot(m) + wy
      wztot(m) = wztot(m) + wz
      do i = 1, 49
        ytot(m,i) = ytot(m,i) + wy*ytemp(i)
        ztot(m,i) = ztot(m,i) + wz*ztemp(i)
        y(m,i) = 0.0
        z(m,i) = 0.0
      enddo
 
      return
 
20    write(*,*)'stack7: called with m=-1 - replacing spectra'



      if( iprint.ge.10 ) then

	  write(*,*)' IVTOT =>'

	  write(*,*)(ivtot(i),i=1,10)

	endif
 
      do m = 1, 10
        write(outd2) m, iv(m), (y(m,i),i=1,49), iv(m), (z(m,i),i=1,49)
      enddo
 
      do i = 1, 10
        if( ivtot(i).eq.0 ) then
          if( iv(i).ne.0 ) then
            nfw(2*i  ) = iv(i)
            nfw(2*i-1) = iv(i)
          endif
        else
          iv(i) = ivtot(i)
          do j = 1, 49
            y(i,j) = ytot(i,j)/wytot(i)
            z(i,j) = ztot(i,j)/wztot(i)
          enddo
        endif
      enddo


      return
 
 
      end
