c     converts time series MARIA files to TS format
c
      real x, y, z, lat, lon
      character maria*3, stat*6, file*12, gfile*15, command*30,
     &          header*80, name*20, line*80
      character fday*6, lday*6, ldayr*6
      character starttime*19
      integer day, yday, dayyr, yy, mm, dd

      integer lnblnk, npts, nfill, i

      real MISDAT
      parameter ( MISDAT = 99999.9 )

      maria = 'smi'
      call tin( 'Give station code for MARIA data', maria )

      if( maria.eq.'con' ) then
c..Contwoyto Lake  CONT  65.75 248.75   72.394 311.295   12.36   73.47
        name = 'Contwoyto Lake'
        lat = 65.75
        lon = 248.75
      else if( maria.eq.'mcm' ) then
c...Fort McMurray   MCMU  56.66 248.79   63.233 315.304    5.49   64.74
        lat = 56.66
        lon = 248.79
        name = 'Fort McMurray'
      else if( maria.eq.'ran' ) then
c...Fort McMurray   RANK  62.82 267.89
        lat = 62.82
        lon = 267.89
        name = 'Rankin Inlet'
c...Fort Simpson    FSIM  61.76 238.77   67.396 300.580    6.84   67.52
      else if( maria.eq.'sim' ) then
        name = 'Fort Simpson'
        lat = 61.76
        lon = 238.77
      else if( maria.eq.'smi' ) then
c...Fort Smith      FSMI  60.02 248.05   66.556 313.205    7.05   67.88
        name = 'Fort Smith'
        lat = 60.02
        lon = 248.05
      else
        write(*,*)'Unrecognized MARIA station: Possible choices:-'
        write(*,*)'con : Contwoyto Lake'
        write(*,*)'mcm : Fort McMurray'
        write(*,*)'ran : Rankin Inlet'
        write(*,*)'sim : Fort Simpson'
        write(*,*)'smi : Fort Smith'
        write(*,*)'For other choices, contact Alan'
        stop
      endif

      stat(1:3) = 'sno'
      call tin( 'Give 3-character prefix to add', stat(1:3) )

      stat = stat(1:3)//maria
      write(*,*) 'Station =>', stat

      file = stat//'as.ts'
      write(*,*)'Opening output ts file >', file
      open(unit=7,file=file,form='unformatted')

      day = 960801
      call iin( 'Give first day (in yymmdd format)', day )
      write( fday, '(i6.6)') day
      yy = day/10000
      mm = day/100 - 100*yy
      dd = mod( day, 100 )
      yday = dayyr( yy, mm, dd )

      starttime = '0000-00-00 00:00:00'
      if( fday(1:1).eq.'9' ) then
        starttime(1:2) = '19'
      elseif( fday(1:1).eq.'0' ) then
        starttime(1:2) = '20'
      else
        write(*,*)'Unrecognized decade: day(1:2) = ', fday(1:2)
        stop
      endif
      write(starttime( 3: 4),'(i2.2)') yy
      write(starttime( 6: 7),'(i2.2)') mm
      write(starttime( 9:10),'(i2.2)') dd


c...write comment block
      line = '#'
      write(7) line
      line = '# Time series file:'
      write(7) line
      line = '#'
      write(7) line
      line = '# MARIA magnetometer site: '//name
      write(7) line
      line = '#'
      write(7) line
      line = '# 3-component magnetic fields recorded at a'
      write(7) line
      line = '# MARIA magnetometer site in northern Canada'
      write(7) line
      line = '#'
      write(7) line

c...write information block
      line = '>STATION   :'//stat
      write(7) line
      line = '>INSTRUMENT:'
      write(line(13:20),'(i8.3)') 0
      write(7) line
      line = '>WINDOW    :'//stat//'as'
      write(7) line
      line = '>LATITUDE  :'
      write(line(13:20),'(f8.4)') lat
      write(7) line
      line = '>LONGITUDE :'
      write(line(13:20),'(f8.4)') lon
      write(7) line
      line = '>ELEVATION :'
      write(line(13:20),'(f8.4)') 0.0
      write(7) line
      line = '>COORD_SYS :TRUE NORTH'
      write(7) line
      line = '>FORM      :BINARY'
      write(7) line
      line = '>FORMAT    :UNFORMATTED'
      write(7) line
      line = '>NCHAN     :   3'
      write(7) line

      line = '>CHAN_1    :HX'
      write(7) line
      line = '>SENSOR_1  : 0'
      write(7) line
      line = '>AZIM_1    : 0.'
      write(7) line
      line = '>UNITS_1   :nT'
      write(7) line
      line = '>GAIN_1    :1.0'
      write(7) line

      line = '>CHAN_2    :HY'
      write(7) line
      line = '>SENSOR_2  : 0'
      write(7) line
      line = '>AZIM_2    :90.'
      write(7) line
      line = '>UNITS_2   :nT'
      write(7) line
      line = '>GAIN_2    :1.0'
      write(7) line

      line = '>CHAN_3    :HZ'
      write(7) line
      line = '>SENSOR_3  : 0'
      write(7) line
      line = '>AZIM_3    : 0.'
      write(7) line
      line = '>UNITS_3   :nT'
      write(7) line
      line = '>GAIN_3    :1.0'
      write(7) line

      line = '>STARTTIME :' // starttime
      write(7) line
      line = '>T_UNITS   :s'
      write(7) line
      line = '>DELTA_T   :   5.0000'
      write(7) line
      line = '>MIS_DATA  :'
      write(line(13:22),'(f10.1)') MISDAT
      write(7) line



      file( 7: 9) = '5s.'
      file(10:12) = maria
      command(1:) = 'gunzip '

      do while( .TRUE. )
        call mmdd( yy, yday, mm, dd )
        write( file(1:6),'(3i2.2)') yy, mm, dd
        gfile = file//'-gz'
        command = 'gunzip '//gfile
        call system( command )
c        write(*,*)'Unzipped file ',gfile

        open(unit=4,file=file,status='old',err=9999)

        read(4,'(a)') header
        write(*,*)'Header >',header(1:lnblnk(header))
        read(header(36:40),'(i5)') npts
        if( npts.gt.17280 ) then
          npts  = 17280
          nfill = 0
        else if( npts.lt.17280 ) then
          write(*,*)'...short file, npts =', npts
          nfill = 17280 - npts
        endif

        do i = 1, npts
          read(4,*) x, y, z
          write(7)  x, y, z
        enddo

        do i = 1, nfill
          write(7) MISDAT, MISDAT, MISDAT
        enddo

        command = 'gzip -S "-gz" '//file
        call system( command )
c        write(*,*)'Rezipped file ',gfile

        yday  = yday+1
        ldayr = file(1:6)
      enddo

9999  write(*,*)'Could not find file ',gfile,' to unzip'
      write(*,*)'End day >', ldayr

      stop
      end
