      program obs2ts

c     converts time series CANADIAN GEOMAGNETIC OBSERVATORY files to TS format

      implicit none

      real MISDAT
      parameter ( MISDAT = 99999.9 )

      real x, y, z, lat, lon, mfactor
      character obs*3, stat*6, file*12, name*20
      character*80 line
      integer day, fday, lday, yday, yy, mm, dd, dddyy, fdddyy, ldddyy
      integer i, ifile, shh, smm, sss, ehh, emm, ess
      integer ix(17280), iy(17280), iz(17280), header(128)
      integer temp1(128), temp2(128)

      obs = 'ykc'
      call tin( 'Give station code for OBSERVATORY data', obs )
      call shiftup( obs )

      if( obs.eq.'YKC' ) then
c..Yellowknife station
        name = 'Yellowknife'
        lat = 62.82
        lon = 245.518

      else if( obs.eq.'MEA' ) then
c..Meanook station
        name = 'Meanook'
        lat = 54.616
        lon = 246.653

      else
        write(*,*)' '
        write(*,*)'Unrecognized OBSERVATORY station: Possible choices:-'
        write(*,*)'YKC : Yellowknife'
        write(*,*)'MEA : Meanook'
        write(*,*)'For other choices, contact Alan'
        stop
      endif

      stat(1:3) = 'sno'
      call tin( 'Give 3-character prefix to add', stat(1:3) )

      call shiftdown( obs )
      stat = stat(1:3)//obs
      write(*,*) 'Station =>', stat

      file = stat//'as.ts'
      call tin('Give output ts filename', file )
      write(*,*)'Opening output ts file >', file
      open(unit=7,file=file,form='unformatted')

      fdddyy = 21196
      call iin( 'Give first day (in dddyy format)', fdddyy )
      yy = mod( fdddyy, 100 )
      yday = fdddyy/100
      call mmdd( yy, yday, mm, dd )
      fday = 10000*yy + 100*mm + dd
      write(*,*)'fdddyy of', fdddyy,'   convert to', fday
      dddyy = fdddyy

      ldddyy = fdddyy
      call iin( 'Give last  day (in dddyy format)', ldddyy )
      yy = mod( ldddyy, 100 )
      yday = ldddyy/100
      call mmdd( yy, yday, mm, dd )
      lday = 10000*yy + 100*mm + dd
      write(*,*)'ldddyy of', ldddyy,'   convert to', lday

c...write comment block
      line = '#'
      write(7) line
      line = '# Time series file:'
      write(7) line
      line = '#'
      write(7) line
      line = '# Canadian Observatory magnetometer site: '//name
      write(7) line
      line = '#'
      write(7) line
      line = '# 3-component magnetic fields recorded at a'
      write(7) line
      line = '# magnetometer site in Canada'
      write(7) line
      line = '#'
      write(7) line

c...write information block
      line = '>STATION   :'//stat
      write(7) line
      line = '>INSTRUMENT:'
      write(line(13:20),'(i8.3)') 0
      write(7) line
      line = '>WINDOW    :'//file(1:8)
      write(7) line
      line = '>LATITUDE  :'
      write(line(13:20),'(f8.4)') lat
      write(7) line
      line = '>LONGITUDE :'
      write(line(13:20),'(f8.4)') lon
      write(7) line
      line = '>ELEVATION :'
      write(line(13:20),'(f8.4)') 0.0
      write(7) line

      line = '>COORD_SYS :TRUE NORTH'
      write(7) line

      line = '>FORM      :BINARY'
      write(7) line
      line = '>FORMAT    :UNFORMATTED'
      write(7) line
      line = '>NCHAN     :   3'
      write(7) line

      line = '>CHAN_1    :HX'
      write(7) line
      line = '>SENSOR_1  : 0'
      write(7) line
      line = '>AZIM_1    : 0.'
      write(7) line
      line = '>UNITS_1   :nT'
      write(7) line
      line = '>GAIN_1    :1.0'
      write(7) line

      line = '>CHAN_2    :HY'
      write(7) line
      line = '>SENSOR_2  : 0'
      write(7) line
      line = '>AZIM_2    :90.'
      write(7) line
      line = '>UNITS_2   :nT'
      write(7) line
      line = '>GAIN_2    :1.0'
      write(7) line

      line = '>CHAN_3    :HZ'
      write(7) line
      line = '>SENSOR_3  : 0'
      write(7) line
      line = '>AZIM_3    : 0.'
      write(7) line
      line = '>UNITS_3   :nT'
      write(7) line
      line = '>GAIN_3    :1.0'
      write(7) line

      line = '>STARTTIME :'
      write(line(13:18),'(i6)') fday
      line(19:24) = '000000'
      write(7) line

      line = '>ENDTIME   :'
      write(line(13:18),'(i6)') lday+1
      line(19:24) = '000000'
      write(7) line

      line = '>T_UNITS   :s'
      write(7) line
      line = '>DELTA_T   :   5.0000'
      write(7) line
      line = '>MIS_DATA  :'
      write(line(13:),* ) MISDAT
      write(7) line


      call shiftup( obs )
      file(1:3) = obs
      file( 9:12) = '.05s' 

      do while( .TRUE. )
        write( file(4:8),'(i5)') dddyy

        open(unit=4,file=file,status='old',err=9999)
        write(*,*)'Opened file ', file
        ifile = ifile+1

        read(4,'(128i8)') header
        if( ifile.eq.1 ) then
          write(*,*)'IAGA station           >', header(1)
          write(*,*)'Data set identifier    >', header(2)
          write(*,*)'Day of month           >', header(3)
          write(*,*)'Number of month        >', header(4)
          write(*,*)'Year                   >', header(5)
          write(*,*)'Sequential day of year >', header(6)
          write(*,*)'Start hour of data (UT)>', header(7)
          write(*,*)'Start min  of data (UT)>', header(8)
          write(*,*)'Start sec  of data (UT)>', header(9)
          write(*,*)'End   hour of data (UT)>', header(10)
          write(*,*)'End   min  of data (UT)>', header(11)
          write(*,*)'End   sec  of data (UT)>', header(12)
          write(*,*)'Sampling interval (s)  >', header(13)
          write(*,*)'X component            >', header(14)
          write(*,*)'Y component            >', header(15)
          write(*,*)'Z component            >', header(16)
          write(*,*)'D component            >', header(17)
          write(*,*)'H component            >', header(18)
          write(*,*)'F component            >', header(19)
          write(*,*)'I component            >', header(20)
          write(*,*)'Other component        >', header(21)
          write(*,*)'Multiplication factor  >', header(22)
          write(*,*)'Temp sample interval   >', header(23)
        endif

        mfactor = float(header(22))

        shh = header(7)
        smm = header(8)
        sss = header(9)
        if( shh.ne.0 .or. smm.ne.0 .or. sss.ne.0 ) then
          write(*,*)'Doesn"t start properly! shh, smm, sss =',
     &              shh, smm, sss
          write(*,*)'Filling day with MIS_DATA marks'
          do i = 1, 17280
            write(7) MISDAT, MISDAT, MISDAT
          enddo
          goto 10
        endif

        ehh = header(10)
        emm = header(11)
        ess = header(12)
        if( ehh.ne.23 .or. emm.ne.59 .or. ess.ne.55 ) then
          write(*,*)'Doesn"t end properly! ehh, emm, ess =',
     &              ehh, emm, ess
          stop
        endif

c...read in the two temperature records
        read(4,'(128i8)') temp1
        read(4,'(128i8)') temp2

c...read in the 5 s data 
c        total no. of pts/channel = 12*60*24 = 17280
c        total no. of pts         = 17280*3  = 51840
c        total no. of records/channel = 17280/128  = 135

        read(4,'(128i8)') ix
        read(4,'(128i8)') iy
        read(4,'(128i8)') iz

        do i = 1, 17280
          x = float(ix(i))/mfactor
          y = float(iy(i))/mfactor
          z = float(iz(i))/mfactor
          write(7) x, y, z
        enddo

10      dddyy = dddyy+100
        if( dddyy .gt. ldddyy ) goto 20
      enddo


20    write(*,*)'Last day requested written out'
      stop

9999  write(*,*)'Could not find file ',file
      write(*,*)'End day >',day-1

      stop
      end
