      program obs_can5sc2ts

c     converts time series CANADIAN GEOMAGNETIC OBSERVATORY files that have
c     been converted using Garret Jansen van Beek's "can_5s" code to TS format

      implicit none

      include 'CTRLBLK.inc'
      include 'DEFAULTS.inc'

      real x(17280), y(17280), z(17280), lat, lon, elev
      character obs*3, stat*6, file*12, name*20
      character*80 line
      character*19, thistime, lasttime
      character*19 stime, etime
      integer tdiff, sdiff, month, day
      integer day, fday, lday, yday, yy, mm, dd, dddyy, fdddyy, ldddyy
      integer i, ii, datagap, ifile, ihour, imin, isec, lnblnk

      iprint = 0
      call iin( 'Give print level', iprint )

      obs = 'ykc'
      call tin( 'Give station code for OBSERVATORY data', obs )
      call shiftup( obs )

      if( obs.eq.'YKC' ) then
c..Yellowknife station
        name = 'Yellowknife'
        lat = 62.82
        lon = 245.518
        elev = 198.

      else if( obs.eq.'MEA' ) then
c..Meanook station
        name = 'Meanook'
        lat = 54.616
        lon = 246.653
        elev = 700.

      else if( obs.eq.'RES' ) then
c..Resolute Bay station
        name = 'Resolute'
        lat = 74.690
        lon = 265.105
        elev = 30.

      else if( obs.eq.'BLC' ) then
c..Baker Lake station
        name = 'Baker Lake'
        lat = 64.318
        lon = 263.988
        elev = 30.

      else
        write(*,*)' '
        write(*,*)'Unrecognized OBSERVATORY station: Possible choices:-'
        write(*,*)'YKC : Yellowknife'
        write(*,*)'MEA : Meanook'
        write(*,*)'BLC : Baker Lake'
        write(*,*)'RES : Resolute Bay'
        write(*,*)'For other choices, contact Alan'
        stop
      endif

      call shiftdown( obs )

      stat(1:3) = 'sno'
      call tin( 'Give 3-character prefix to add', stat(1:3) )

      call shiftdown( obs )
      stat = stat(1:3)//obs
      write(*,*) 'Station =>', stat

      file = stat//'as.ts'
      call tin('Give output ts filename', file )
      write(*,*)'Opening output ts file >', file
      open(unit=7,file=file,form='unformatted')

      fdddyy = 19798
      call iin( 'Give first day (in dddyy format)', fdddyy )
      yy = mod( fdddyy, 100 )
      yday = fdddyy/100
      call mmdd( yy, yday, mm, dd )
      fday = 10000*yy + 100*mm + dd
      write(*,*)'fdddyy of', fdddyy,'   converted to', fday
      dddyy = fdddyy

      ldddyy = 36597
      call iin( 'Give last  day (in dddyy format)', ldddyy )
      yy = mod( ldddyy, 100 )
      yday = ldddyy/100
      call mmdd( yy, yday, mm, dd )
      lday = 10000*yy + 100*mm + dd
      write(*,*)'ldddyy of', ldddyy,'   converted to', lday

c...write comment block
      line = '#'
      write(7) line
      line = '# Time series file:'
      write(7) line
      line = '#'
      write(7) line
      line = '# Canadian Observatory magnetometer site: '//name
      write(7) line
      line = '#'
      write(7) line
      line = '# 3-component magnetic fields recorded at a'
      write(7) line
      line = '# magnetometer site in Canada'
      write(7) line
      line = '#'
      write(7) line

c...write information block
      line = '>STATION   :'//stat
      write(7) line
      line = '>INSTRUMENT:'
      write(line(13:20),'(i8.3)') 0
      write(7) line
      line = '>WINDOW    :'//file(1:8)
      write(7) line
      line = '>LATITUDE  :'
      write(line(13:20),'(f8.4)') lat
      write(7) line
      line = '>LONGITUDE :'
      write(line(13:20),'(f8.4)') lon
      write(7) line
      line = '>ELEVATION :'
      write(line(13:20),'(f8.1)') elev
      write(7) line

      line = '>COORD_SYS :TRUE NORTH'
      write(7) line

      line = '>FORM      :BINARY'
      write(7) line
      line = '>FORMAT    :UNFORMATTED'
      write(7) line
      line = '>NCHAN     :   3'
      write(7) line

      line = '>CHAN_1    :HX'
      write(7) line
      line = '>SENSOR_1  : 0'
      write(7) line
      line = '>AZIM_1    : 0.'
      write(7) line
      line = '>UNITS_1   :nT'
      write(7) line
      line = '>GAIN_1    :1.0'
      write(7) line

      line = '>CHAN_2    :HY'
      write(7) line
      line = '>SENSOR_2  : 0'
      write(7) line
      line = '>AZIM_2    :90.'
      write(7) line
      line = '>UNITS_2   :nT'
      write(7) line
      line = '>GAIN_2    :1.0'
      write(7) line

      line = '>CHAN_3    :HZ'
      write(7) line
      line = '>SENSOR_3  : 0'
      write(7) line
      line = '>AZIM_3    : 0.'
      write(7) line
      line = '>UNITS_3   :nT'
      write(7) line
      line = '>GAIN_3    :1.0'
      write(7) line

      line = '>STARTTIME :'
      write(stime,'(i6)') fday
      stime(7:12) = '000000'
      call convtimev5( stime )
      line(13:) = stime
      write(7) line

      line = '>ENDTIME   :'
      write(etime,'(i6)') lday+1
      etime(7:12) = '000000'
      call convtimev5( etime )
      line(13:) = etime
      write(7) line

      line = '>T_UNITS   :s'
      write(7) line
      line = '>DELTA_T   :   5.0000'
      write(7) line
      line = '>MIS_DATA  :'
      write(line(13:),* ) DEFMISDAT
      write(7) line


      file(1:3) = obs
      file(4:8) = '_5sc.'

      do 500 dddyy = fdddyy, ldddyy, 100

        call mmdd( mod(dddyy,100), dddyy/100, month, day )
        write( thistime(1:6),'(3i2.2)') mod(dddyy,100), month, day
        thistime(7:12) = '000000'
        call convtimev5( thistime )

        write( file(9:11),'(i3.3)') dddyy/100
        open(unit=4,file=file,status='old',err=100)
        write(*,'(2a,$)')'Opened file: ', file(1:11)
c        write(*,'(2a)')'Opened file: ', file(1:11)
        ifile = ifile+1
        goto 200

c...no file for that day. write out missing data records
100     write(*,*)'Missing day:', dddyy/100,
     &            '  Filling with 17280 missing data markers'
        do i = 1, 17280
          write(7) DEFMISDAT, DEFMISDAT, DEFMISDAT
        enddo
        goto 500

c...read in the 5 s data 

c...check new day starts at midnight
200     i = 1
        read(4,'(a)') line
        read(line(15:16),*) ihour
        read(line(18:19),*) imin
        read(line(21:22),*) isec

        if( iprint.ge.1 ) then
          write(*,*)'First record read in >',line(1:lnblnk(line))
        endif

        if( ihour.lt.0 .or. ihour.gt.23 .or.
     &      imin .lt.0 .or. imin .gt.59 .or.
     &      isec .lt.0 .or. isec .gt.59 ) then
          write(*,*) 'FILE CORRUPT !!!!'
          write(*,*) 'ihour =', ihour
          write(*,*) 'imin  =', imin
          write(*,*) 'isec  =', isec
          stop
        endif

        if( ihour+imin+isec .ne. 0 ) then
          write(*,*)'Day not started at midnight'
          write(*,*)'Day, hour, min, sec =',dddyy/100, ihour, imin, isec
          write(thistime(12:13),'(i2.2)') ihour
          write(thistime(15:16),'(i2.2)') imin
          write(thistime(18:19),'(i2.2)') isec
          sdiff = tdiff( lasttime, thistime )
          datagap = sdiff/5 -1
          write(*,*)'Data gap of length:', datagap
          write(*,*)'thistime: ', thistime, ' lasttime: ', lasttime
          do i = 1, datagap
            x(i) = DEFMISDAT
            y(i) = DEFMISDAT
            z(i) = DEFMISDAT
          enddo
        else
          read(line(23:32),*) x(1)
          read(line(33:42),*) y(1)
          read(line(43:52),*) z(1)
        endif
        i = i+1
        lasttime = thistime

        do while( i.le.17280 )
          read(4,'(a)',end=300) line

          if( iprint.ge.10 ) then
            write(*,*)'Next  record read in >',line(1:lnblnk(line))
          endif

          read(line(15:16),*,err=901) ihour
          read(line(18:19),*,err=901) imin
          read(line(21:22),*,err=901) isec

          if( ihour.lt.0 .or. ihour.gt.23 .or.
     &        imin .lt.0 .or. imin .gt.59 .or.
     &        isec .lt.0 .or. isec .gt.59 ) then
            write(*,*)'FILE CORRUPT !!!!'
            write(*,*) 'ihour =', ihour
            write(*,*) 'imin  =', imin
            write(*,*) 'isec  =', isec
          endif

          write(thistime(12:13),'(i2.2)') ihour
          write(thistime(15:16),'(i2.2)') imin
          write(thistime(18:19),'(i2.2)') isec

          sdiff = tdiff( lasttime, thistime )

          if( sdiff.ne.5 ) then
            datagap = sdiff/5 -1
            write(*,*)
            write(*,*)'Data gap of length:', datagap
            write(*,*)'thistime: ', thistime, ' lasttime: ', lasttime
            do ii = i, i+datagap
              x(ii) = DEFMISDAT
              y(ii) = DEFMISDAT
              z(ii) = DEFMISDAT
            enddo
            i = i + datagap
          endif


          read(line(23:32),*,err=902) x(i)
          read(line(33:42),*,err=902) y(i)
          read(line(43:52),*,err=902) z(i)
          lasttime = thistime
          i = i+1
        enddo
        do i = 1, 17280
          write(7) x(i), y(i), z(i)
        enddo
        write(*,*) '  wrote out 17280 data points'
        goto 500

c...Data not complete to end of day. Fill with missing data markers
300     write(*,*)'  File ended before end-of-day'
        write(*,*)'last time with data = ', lasttime
        write(*,*)'No. of missing data markers to write = ', 17280-i+1
        do while( i.le.17280 )
          write(7) DEFMISDAT, DEFMISDAT, DEFMISDAT
          i = i+1
        enddo

500   continue


      write(*,*)'Last day requested written out'
      stop

901   write(*,*)'Time error in file: lasttime: ', lasttime
      stop

902   write(*,*)'Date error in file: lasttime: ', lasttime
      stop

      end
