#!/bin/csh  -f
#
#  lims_crscov
#
#  computes cross-covariance between one series and another
#

onintr exit

set old=$cwd

if ( $#argv != 0 ) then
  set back = y
else
  set back = n
endif


start:
set root = ` `

set stn      = `cat .lims.stn`
set dbase    = `cat .lims.dbase`
set stn_dir  = `cat .lims.stn_dir`
set lims_dir = `cat .lims.lims_dir`

cd $stn_dir

window:
  echo  ' ' 
  echo  '  WINDOWS FOUND IN: '$stn_dir
  echo  ' '
  ls *.ts
  echo  ' ' 
  echo ' Enter Run number & Window (no extension) (e.g. a1) '
window_select:
  echo ' '
  echo -n 'Window? ===> '$root
  set window=$<
  set window=$stn$window
  set tmp=$window'.ts'
  if( -e $tmp ) then
    echo 'Window selected =>'$window'.ts'
  else
    echo 'Window not found; re-select'
    goto window_select
  endif

  
remote:
  cd $lims_dir$dbase
  echo ' '
  echo 'Determining possible time overlap windows'
  echo 'windowoverlap   <<END  >.lims.remote'  > .lims_crscov
  echo '-1'                                   >> .lims_crscov
  echo $window                                >> .lims_crscov
  echo 'T'                                    >> .lims_crscov #same delta
  echo 'T'                                    >> .lims_crscov #reject spliced
  echo 'END'                                  >> .lims_crscov
  chmod +x .lims_crscov
  .lims_crscov
  cat .lims.remote

  echo  ' ' 
  echo ' Enter station number to cross-covariate with (last 3 characters)'
remote_select:
  echo ' '
  echo -n 'Remote? ===> '$root
  set remote=$<
  set remote=$dbase$remote
  if( -e $remote ) then
    echo 'Station selected =>'$remote
  else
    echo 'Station not found; re-select'
    goto remote_select
  endif

  grep $remote .lims.remote > .lims.rlist

  if !( -e .lims.rlist ) then
    echo 'Incorrect station listed! Choose again:'
    cat .lims.remote
    goto remote_select
  endif

  set nstat=`wc -l .lims.rlist | awk '{print $1}'`

  if ( $nstat == 1 ) then
    set rrwindow = `grep $remote .lims.remote | awk -F: '{print $1}'`
  else
    echo 'More than one overlapping window for this station:'
    cat .lims.rlist
    echo ' '
    echo -n 'Remote window (last two characters, e.g. a1) ? ===> '$root
    set rrwindow=$<
    set rrwindow = `grep $rrwindow .lims.rlist | awk -F: '{print $1}'`
  endif

  echo 'Remote window selected =>'$rrwindow'.ts'

times:
  set line = `grep $rrwindow .lims.rlist`
  echo $line > .lims.remote
  set stime = `awk -F=  '{print $2}' .lims.remote | awk -F- '{print $1}'`

  echo ' '
stime_select:
  echo    'Give start time: default >'$stime''
  echo -n 'Start time?           ===>'$root
  set ans=$<
  if !( $ans == "" ) then
    set stime = $ans
  endif

duration:
  echo ' '
  set duration = 2400
  echo    'Give duration (points): default >'$duration
duration_select:
  echo -n 'Duration?                    ===>'$root
  set ans=$<
  if ( $ans != "" ) then
    set duration = $ans
  endif


crscov:
  cd $old
  echo 'crscov: window='$window' with='$rrwindow' stime='$stime' duration='$duration >> lims.log
  cd $lims_dir$dbase
if( $back == y ) then
  echo 'crscov > .lims.crscov.out <<END'   > .lims_crscov
else
  echo 'crscov      << END'     > .lims_crscov
endif
  echo '-1'                    >> .lims_crscov   #iprint level
  echo $window                 >> .lims_crscov   #window
  echo $stime                  >> .lims_crscov   #start time
  echo $duration               >> .lims_crscov   #duration
  echo $rrwindow               >> .lims_crscov   #rr window
  echo ' '                     >> .lims_crscov   #start time correct
  echo ' '                     >> .lims_crscov   #points correct
  echo ' '                     >> .lims_crscov   #o/p filename correct
  echo ' '                     >> .lims_crscov   #best guess correct correct
  echo ' '                     >> .lims_crscov   #delta-t correct
  echo 'END'                   >> .lims_crscov

  chmod +x .lims_crscov
  if( $back == y ) then
    echo 'Backgrounding crscov job...'
    .lims_crscov &
  else
    .lims_crscov
  endif







exit:
  cd $old
