#!/bin/csh  -f
#
#  lims_mtc
#
#  plots data from a station
#

onintr exit

set old=$cwd


if ( $#argv == 0 ) then
  set dev = 3
  set back = n
else
  set dev = 1
  if ( $#argv == 2 ) then
    set back = y
  else
    set back = n
  endif
endif


start:
set root = ` `

set stn      = `cat .lims.stn`
set dbase    = `cat .lims.dbase`
set stn_dir  = `cat .lims.stn_dir`
set lims_dir = `cat .lims.lims_dir`

#---check existence of file mtc.mtp
cd $lims_dir$dbase
if !( -e mtc.mtp ) then
  echo ' '
  echo 'plot parameter file mtc.mtp not in database directory '$lims_dir$dbase
  echo ' '
  echo 'Cannot run mtc plot program until mtc.mtp exists -- exiting'
  goto exit
endif


cd $stn_dir

  echo  ' ' 
  echo  '  RESPONSE FILES FOUND IN: '$stn_dir
  echo  ' '
  ls *.dat

  if( -e $stn'r.dat' || -e $stn'cr.dat' ) set r = r
  if( -e $stn'l.dat' || -e $stn'cl.dat' ) set r = l

  if( -e $stn'r.dat'  &&  -e $stn'l.dat' ) then
    echo ' '
    echo 'Both local & remote response files found ... plot local or remote ?'
response_select:
    echo -n 'Response (l/r) =>'$root
    set r=$<
    if( $r != 'r'  &&  $r != 'l' ) then
      echo 'Incorrect response -- re-enter'
      goto response_select
    endif
  endif

  if ( -e $stn$r'.dat' ) then
    set j = 'j'
  else
    set j = 'x'
  endif

  if ( -e $stn$r'.dat' && -e $stn'c'$r'.dat' ) then
    echo ' '
    echo 'Both jones & chave response files found ... plot jones, chave, or both ?'
chave_select:
    echo -n 'Response (j/c/b) =>'$root
    set j=$<
    if ( $j != 'j' &&  $j != 'c' &&  $j != 'b' ) then
      echo 'Incorrect response -- re-enter'
      goto response_select
    endif
  endif

if( $dev == 1 ) then
title:
  echo ' '
  echo 'Enter the title you wish at the base of the plot'
  echo -n 'Title =>'
  set title = $<
endif

cd $lims_dir$dbase

if( $dev == 3 ) then
  echo 'Press <RETURN> in graphics window when finished viewing...'
endif

  echo 'mtc -udev '$dev' > /dev/null  <<END'     > .lims_mtc
if( $dev == 1 ) then
  echo ' '                    >> .lims_mtc   #default b/w pallette
endif
  echo 'x'                    >> .lims_mtc   #no models
  echo 'F'                    >> .lims_mtc   #no identify curves
  echo 'T'                    >> .lims_mtc   #pyx phases in 1st quad
  echo '0.0'                  >> .lims_mtc   #minimum weighting
  echo 'T'                    >> .lims_mtc   #reject bad points
if( $j == 'j' || $j == 'b' ) then
  echo $stn'/'$stn$r'.dat'    >> .lims_mtc   #file
else
  echo $stn'/'$stn'c'$r'.dat' >> .lims_mtc   #file
endif
  echo ' '                    >> .lims_mtc   #station
  echo ' '                    >> .lims_mtc   #RXY
  echo 'y'                    >> .lims_mtc   #yes - another response
  echo ' '                    >> .lims_mtc   #file
  echo ' '                    >> .lims_mtc   #station
  echo ' '                    >> .lims_mtc   #RYX
if( $j == 'b' ) then
  echo 'y'                    >> .lims_mtc   #yes - another response
  echo $stn'/'$stn'c'$r'.dat' >> .lims_mtc   #file
  echo ' '                    >> .lims_mtc   #station
  echo 'RXY'                  >> .lims_mtc   #RXY
  echo 'y'                    >> .lims_mtc   #yes - another response
  echo ' '                    >> .lims_mtc   #file
  echo ' '                    >> .lims_mtc   #station
  echo 'RYX'                  >> .lims_mtc   #RYX
endif
  echo 'n'                    >> .lims_mtc   #no to another response
if( $dev == 1 ) then
  echo $title                 >> .lims_mtc   #text for base of plot
else
  echo ' '                    >> .lims_mtc   #text for base of plot
endif
  echo ' '                    >> .lims_mtc   #mtc.mtp
  echo ' '                    >> .lims_mtc   #models
  echo 'END'                  >> .lims_mtc   #END
if( $dev == 1 ) then
  echo 'lpr -Pps mtc.ps'      >> .lims_mtc   #print plot
endif

  chmod +x .lims_mtc
  if( $back == y ) then
    echo 'Backgounding print job...'
    .lims_mtc &
  else
    .lims_mtc
  endif

exit:
  cd $old

