#!/bin/csh -f

onintr exit

#
# 4.00 Hz, 1.00 Hz, 5.0 second and 30.0 second bands for data processing
# do_bands set to 1 if band is to be reprocessed, 0 if not
# do_cas  set to 1 if tscascade is to be used, 0 if rrrmt in that band
#
set bands = ( '4.000000$' '1.000000$' '0.200000$' '0.033333$' )
set do_bands = (   0        1       1      1 )
set do_cas   = (   0        1       1      1 )
#
#
# this script can be run any number of times in parallel with the
# following caveat:
#   if you do rrrmt processing in a band you must ensure multiple
#   instances of processing do not occur simultaneously for the same 
#   site
#
# Sift through all files in project windows file for remote processing
#
# D.E. Boerner July 1994
# mods         Oct  1995
#
# v4.x mods: agj may 1997
# v5.x mods: agj march 1999
#

if ($#argv == 0) then
  echo 'Automatic .ts processing script. '
  echo ' '
  echo 'Sifts through all files in windows.xxx file for processing '
  echo ' '
  echo '* IGNORES ALL SPLICED DATA with the following convention'
  echo ' '
  echo 'Assumes 1s files for  1 sec data' 
  echo '        as files for  5 sec data'
  echo '        xs files for 30 sec data'
  echo ' '
  echo ' You can keep the intermediate cascade files by setting keep=1'
  echo ' You can debug the script by setting debug=1'
  echo ' '
  echo 'USAGE: lims_remote_all <site_number> <MT or GDS> <strike> <keep>/<debug>'
  echo 'i.e. : lims_remote_all 001 mt 0 keep/debug' 
  echo ' '
  exit(0)
else
  set num = $argv[1]
  shift
endif

#
#...set MT or GDS processing
#
if ($#argv == 0) then
  echo -n "MT or GDS ? "
  set mode = ($<) 
else
  set mode = $argv[1]
  shift
endif

#
#...set strike angle
#
if ($#argv == 0) then
  echo -n "Strike angle ? "
  set p_coord = ($<) 
 else
  set p_coord = $argv[1]
  shift
endif
set strike = $p_coord

#
# see if "keep" or "debug" set
#
set keep  = 0
set debug = 0
if ($#argv != 0) then
  set param = $argv[1]
  if( $param == keep ) then
    set keep  = 1
  else if( $param == debug ) then
    set debug = 1
  endif
  shift
endif
if ($#argv != 0) then
  set param = $argv[1]
  if( $param == keep ) then
    set keep  = 1
  else if( $param == debug ) then
    set debug = 1
  endif
endif

if ( $debug == 1 ) then
  echo '------debug set'
endif

if ( $keep == 1 ) then
  echo '------keep set'
endif


set ib     = 1
set srvy   = `echo $cwd | awk -F/ '{print $NF}'` 
set wf     = 'windows.'$srvy
set site   = $srvy$num
set in     = lims_remote_file_iput_$$
set out    = lims_remote_file_oput_$$
rm -f $in $out

#
#...loop around bands...........................................................
#
foreach band ( $bands )
  set bd = `echo $band | cut -c1-8`
  echo '----------------------NEW BAND: '$bd' Hz -------------------------'

  if ( ! $do_bands[$ib] ) then
    echo 'Band not to be processed...'
    goto new_band
  endif

  grep $site $wf | grep $bd  >& /dev/null
  if ( $status ) then
    echo 'No files for the station with this band =' $bd
    goto new_band
  endif
#
#
  if ( $band == '1.000000$' ) then
    set files = `grep $site'1s' $wf | awk '{print $1}'`
    if( -e $site/$site'1s.ts') then
      echo 'spliced 1s file exists'
      set files = `grep $site'1s' $wf | awk '{print $1}'`
    else
      echo 'spliced 1s file does not exist'
      set files = `grep $site $wf | grep -v '#' | grep $bd | awk '{print $1}'`
    endif
  else if ( $band == '0.200000$' ) then
    if( -e $site/$site'as.ts') then
      echo 'spliced as file exists'
      set files = `grep $site'as' $wf | awk '{print $1}'`
    else
      echo 'spliced as file does not exist'
      set files = `grep $site $wf | grep -v '#' | grep $bd | awk '{print $1}'`
    endif
  else
    if( -e $site/$site'xs.ts') then
      echo 'spliced xs file exists'
      set files = `grep $site'xs' $wf | awk '{print $1}'`
    else
      echo 'spliced xs file does not exist'
      set files = `grep $site $wf | grep -v '#' | grep $bd | awk '{print $1}'`
    endif
  endif

  echo 'Station list of files -->'
  echo $files
  set cascade_list = ' '

  foreach file ( $files )
    echo 'File -->' $file

#...run woverlap to get remote overlaps
    echo "-1"   > $in
    echo $file >> $in
    echo "T"   >> $in
    echo "F"   >> $in
    woverlap -1 < $in > $out


    if ( $band == '1.000000$' ) then
      set remote_list = `cat $out | awk -F: '{print $1}'`
    else if ( $band == '0.200000$' ) then
      set remote_list = `cat $out | awk -F: '{print $1}'`
    else if ( $band == '0.033333$' ) then
      set remote_list = `cat $out | awk -F: '{print $1}'`
    endif

    echo 'Remote list of files -->'
    echo $remote_list

#
# form cascade decimates with all available other files (excluding local)
#
    foreach rfile ( $remote_list )

      if ( $debug == 1 ) echo 'Forming cascade decimates with file '$rfile

      set rnum  = `echo $rfile | cut -c4-6`
      set rem   = `echo $rfile | cut -c4-8`

#...get the start and end times
      set wst = `grep $file $wf | cut -c16-34`
      set wen = `grep $file $wf | cut -c36-54`
      set rst = `grep $rfile $wf | cut -c16-34`
      set ren = `grep $rfile $wf | cut -c36-54`

#...compare window start with remote start
      compdate $wst $rst
      if ( $status == 0 ) then
        set start = `echo $wst`
      else
        set start = `echo $rst`
      endif

#...compare window end with remote end
      compdate $wen $ren
      if ( $status == 0 ) then
        set end = `echo $ren`
      else
        set end = `echo $wen`
      endif

      if ( $debug == 1 ) then
        echo 'Window start and end : ', $wst ' - ' $wen
        echo 'Remote start and end : ', $rst ' - ' $ren
      endif

      echo 'Start: '$start    '   End: '$end

#...calculate duration
      echo "0"         >  $in          # print level
      echo $start      >> $in          # start
      echo $end        >> $in          # end
      wlength < $in > $out

      set dur   = `cat $out | awk -F= '{print $2}'`
      set ddur  = `echo $dur | cut -c1-4`
      set hdur  = `echo $dur | cut -c5-6`
      set mdur  = `echo $dur | cut -c7-8`
      set sdur  = `echo $dur | cut -c9-10`


# Start processing cycle

      echo "0"         > $in          # print level
      echo "Y"        >> $in          # remote reference
      echo $mode      >> $in          # MT processing
      echo "Y"        >> $in          # STACK
      echo "Y"        >> $in          # JJANAL
      echo $file      >> $in
      echo $rfile     >> $in
      echo "b"        >> $in          # BINARY local  input
      echo "b"        >> $in          # BINARY remote input
      echo $start     >> $in          # start time
      echo $end       >> $in          # end   time
      echo $p_coord   >> $in          # processing coordinate system


#...set filenames and remove if they exist

      set window = `echo $file | cut -c4-8`
      if ( $mode == 'GDS' || $mode == 'gds') then
        set casfile = $window$rem'_'$mode.cas
      else
        set casfile = $window$rem.cas
      endif

#...add casfile to list
      set cascade_list = ( $cascade_list $casfile )

      set lab = $file' & '$rfile' (RR)'

      echo $lab': Overlap (days-hh:mm:ss) ='$ddur'-'$hdur':'$mdur':'$sdur

#...remove .out & .cas files if they exists
      rm -f $site/$window$rnum.out >& /dev/null
      rm -f $site/$casfile         >& /dev/null
      echo 'removed files....' $site/$window$rnum.out'    and   '$site/$casfile

      if ( $debug == 0 ) then
        tscascade -ask < $in >& $out
      else
        echo 'Input to tscascade.................................'
        cat $in
        echo 'End input..........................................'
        tscascade -ask < $in 
      endif


      mv $site/$window$rnum.cas $site/$casfile
      rm -f $site/$window$rnum.out >& /dev/null
      rm -f $in $out 

new_station:
    end
#...end of rfile loop

  end
#...end of file loop

#...write out cascade files
  rm -f $site/cascade.lst >& /dev/null
  touch $site/cascade.lst
  foreach file ( $cascade_list )
    echo $file >> $site/cascade.lst
  end

#
# restack section - needs filter entry in hardware."srvy" for the site
#
  echo "Starting stacking run..."
  echo "0"     > $in                           # print level
  echo $site  >> $in                           # site
  echo "Y"    >> $in                           # Y - remote reference
  echo $mode  >> $in                           # mode - mt or gds - input param
  if ( $band == '1.000000$' ) then             # minimum coherence
    echo 0.50    >> $in                         # 0.5 for 1 s data
  else if ( $band == '0.200000$' ) then
    echo 0.50    >> $in                         # 0.5 for 1 s data
  else if ( $band == '0.033333$' ) then
    echo 0.25    >> $in                         # 0.25 for 1 s data
  endif
  echo "0.9999" >> $in                         # maximum coherence
  echo "Y"      >> $in
  echo "JKN"    >> $in
  echo " "      >> $in
  echo "Y"      >> $in
  foreach file ( $cascade_list )
    if ( -e $site/$file ) echo $file >> $in
  end
  echo "stop"     >> $in
  echo "$strike"  >> $in
  rm -f $site/$site.xr $site/$site'.xrg' $site/$site'r.dat' $site/$site.out

  if ( $debug == 0 ) then
    tsrestack -ask < $in >& tsrestack_out
  else
    echo 'Input to tsrestack.................................'
    cat $in
    echo 'End input..........................................'
    tsrestack -ask < $in
  endif


#  delete cascade files
  if ( $keep == 0 ) then
    rm $site/*.cas*
    rm $site/cascade.lst
  endif


  if ( $mode == 'GDS' || $mode == 'gds') then
    mv $site/$site'r.dat $site/$site'_'$ib'r'$mode'.dat'
   else
    mv $site/$site'r.dat' $site/$site'_'$ib'r.dat'
  endif
  rm -f $out $in

new_band:
  @ ib = ( $ib + 1 )
end
#
#...ended loop around bands.....................................................
#

goto exit

#
# merge output files together
#
lims_merge $site 
#
#
# plot final data file
#
# mt_plot jfor/$site.dat
#
#
# create .edi file
#
cd jfor
j2edi <<EOD
$site
stop
12
EOD
mv $site.edi ~/edi
echo ' '
echo ' '
echo 'edi file created (in edi directory)'
cd ..
#
# cleanup files
#

rm -f $in $out 
rm -f $site/$site??.out7
rm -f tmp.*
rm -f lims_remote_file*

exit (0)
