#!/bin/csh -f

onintr goto exit

#
# 4.00 Hz, 1.00 Hz, 5.0 second and 30.0 second bands for data processing
# do_bands set to 1 if band is to be reprocessed, 0 if not
# do_cas  set to 1 if tscascade is to be used, 0 if rrrmt in that band
#
set bands = ( '4.000000$' '1.000000$' '0.200000$' '0.033333$' )
set do_bands = (   0        1       1      1 )
set do_cas   = (   0        1       1      1 )
#
#
# this script can be run any number of times in parallel with the
# following caveat:
#   if you do rrrmt processing in a band you must ensure multiple
#   instances of processing do not occur simultaneously for the same 
#   site
#
# Sift through all files in project windows file for remote processing
# Reads in a list of times that are to be processed from files "times.xxx"
# where "xxx" is the project code
#
# A G Jones: Oct 1998
#
if ($#argv == 0) then
  echo 'Automatic .ts processing script. '
  echo ' '
  echo 'Sifts through all files in windows.xxx file for processing'
  echo 'Uses start and end times in file times.prj'
  echo ' '
  echo 'Uses 1s files for  1 sec data' 
  echo '     as files for  5 sec data'
  echo '     xs files for 30 sec data'
  echo ' '
  echo ' You can keep the intermediate cascade files by setting keep=1'
  echo ' '
  echo 'USAGE: lims_remote_list <site_number> <MT or GDS> <strike> [keep] [debug]'
  echo 'i.e. : lims_remote_list 001 mt 0 keep debug' 
  echo ' '
  exit(0)
else
  set num = $argv[1]
  shift
endif

if ($#argv == 0) then
  echo -n "MT or GDS ? "
  set mode = ($<) 
else
  set mode = $argv[1]
  shift
endif

if ($#argv == 0) then
  echo -n "Strike angle ? "
  set p_coord = ($<) 
 else
  set p_coord = $argv[1]
  shift
endif
set strike = $p_coord

set keep  = 0
set debug = 0
if ($#argv != 0) then
  set param = $argv[1]
  if( $param == keep ) then
    set keep  = 1
  else if( $param == debug ) then
    set debug = 1
  endif
  shift
endif
if ($#argv != 0) then
  set param = $argv[1]
  if( $param == keep ) then
    set keep  = 1
  else if( $param == debug ) then
    set debug = 1
  endif
endif

if ( $debug == 1 ) then
  echo '------debug set'
endif

if ( $keep == 1 ) then
  echo '------keep set'
endif

set ptime = `date`
set ptime = `echo $ptime | cut -c4-16,20-28`

set ib = 1
set srvy = `echo $cwd | awk -F/ '{print $NF}'` 
set wf   = 'windows.'$srvy
set site   = $srvy$num
set log    = $site/auto_process.log
set in     = lims_remote_file_iput_$$
set out    = lims_remote_file_oput_$$
rm -f $in $out

set times_file = 'times.'$num
if !( -e $times_file ) then
  echo 'File containing times to used not found >' $times_file
  echo 'create and re-run'
  goto exit
else
  echo 'Opening time file >'$times_file
endif
set times_list = `cat $times_file | grep -v '#'`


grep $num $srvy'site.flt' >& /dev/null
if ( $status != 0 ) then
  echo 'Site '$srvy$num' is not in the flat file '$srvy'site.flt' 
  echo 'You must copy this file from the PC to the SPARC'
  exit(0)
endif
echo ' ' 
#
if ( ! ( -e $log ) ) touch $log

foreach band ( $bands )

  set bd = `echo $band | cut -c1-8`
  echo '----------------------NEW BAND: '$bd' Hz -------------------------'

  if ( ! $do_bands[$ib] ) then
    echo 'Band not to be processed...'    
    goto new_band
  endif

  grep $site $wf | grep $bd  >& /dev/null
  if ( $status )  then
    echo 'No files for the station with this band =' $bd
    goto new_band
  endif

  if ( $debug == 1 ) then
    echo 'Matches in windows file for this band at this site...'
    grep $site $wf | grep $band
  endif

  if ( $band == '1.000000$' ) then
    set files = `grep $site'1s' $wf | grep -v '#' | awk '{print $1}'`
    if( -e $site/$site'1s.ts') then
      echo 'spliced 1s file exists'
      set files = `grep $site'1s' $wf | grep -v '#' | awk '{print $1}'`
    else
      echo 'spliced 1s file does not exist'
      set files = `grep $site $wf | grep -v '#' | grep $bd | awk '{print $1}'`
    endif
  else if ( $band == '0.200000$' ) then
    if( -e $site/$site'as.ts') then
      echo 'spliced as file exists'
      set files = `grep $site'as' $wf | grep -v '#' | awk '{print $1}'`
    else
      echo 'spliced as file does not exist'
      set files = `grep $site $wf | grep -v '#' | grep $bd | awk '{print $1}'`
      echo 'Files = '$files
    endif
  else
    if( -e $site/$site'xs.ts') then
      echo 'spliced xs file exists'
      set files = `grep $site'xs' $wf | grep -v '#' | awk '{print $1}'`
    else
      echo 'spliced xs file does not exist'
      set files = `grep $site $wf | grep -v '#' | grep $bd | awk '{print $1}'`
    endif
  endif

  set nf = `echo $files | grep $site | wc | awk '{print $1}'`

  if ( $nf == '0' ) then
    echo 'No files for this band to process'
    goto new_band
  endif

  set file = $files
  echo 'File to use -->'
  echo $file

 
#...run woverlap to get remote overlaps
    echo "-1"   > $in
    echo $file >> $in
    echo "T"   >> $in
    echo "F"   >> $in
    woverlap -1 < $in > $out
 
 
    if ( $band == '1.000000$' ) then
      set remote_list = `cat $out | awk -F: '{print $1}'`
    else if ( $band == '0.200000$' ) then
      set remote_list = `cat $out | awk -F: '{print $1}'`
    else if ( $band == '0.033333$' ) then
      set remote_list = `cat $out | awk -F: '{print $1}'`
    endif
 
    echo 'Remote list of files -->'
    echo $remote_list

  set cascade_list = ' '

  if ( $debug == 1 ) then 
    echo 'Times list -->'
    echo $times_list
  endif

foreach file ( $files )
#...get the start and end times
  set wst = `grep $file $wf | cut -c16-34`
  set wen = `grep $file $wf | cut -c36-54`

  echo 'Station: ' $file'  Recording times: ' $wst ' - ' $wen

 foreach rfile ( $remote_list )
  set i = 1

#...get the remote start and end times
   set rst = `grep $rfile $wf | cut -c16-34`
   set ren = `grep $rfile $wf | cut -c36-54`

  echo 'Remote : ' $rfile'  Recording times: ' $rst ' - ' $ren

#...test whether the window start is before of after the remote end
    compdate $wst $ren
    if ( $status == 0 ) then
      echo 'Window start > remote end: next remote'
      goto new_remote
    endif

#...test whether the window end is before of after the remote start
    compdate $wen $rst
    if ( $status == 1 ) then
      echo 'Window end < remote start: next remote'
      goto new_remote
    endif

  foreach times ( $times_list )

#...skip over blank and hash lines in input times file
    set firstchars =  `echo $times | cut -c1-2`
    if( $firstchars != '19' && $firstchars != '20' ) goto new_time

#...convert times into v5 format
    set tstart = `echo $times | cut -c1-14`
    set tend   = `echo $times | cut -c16-29`

    set yy = `echo $tstart | cut -c1-4`
    set mm = `echo $tstart | cut -c5-6`
    set dd = `echo $tstart | cut -c7-8`
    set hh = `echo $tstart | cut -c9-10`
    set mn = `echo $tstart | cut -c11-12`
    set ss = `echo $tstart | cut -c13-14`
    set tstart = `echo $yy'-'$mm'-'$dd' '$hh':'$mn':'$ss`

    set yy = `echo $tend | cut -c1-4`
    set mm = `echo $tend | cut -c5-6`
    set dd = `echo $tend | cut -c7-8`
    set hh = `echo $tend | cut -c9-10`
    set mn = `echo $tend | cut -c11-12`
    set ss = `echo $tend | cut -c13-14`
    set tend = `echo $yy'-'$mm'-'$dd' '$hh':'$mn':'$ss`

    if( $debug == 1 ) then
      echo ' '
      echo 'Times:' $tstart - $tend
    endif


# form cascade decimates with all available other files (excluding local)

    set rnum  = `echo $rfile | cut -c4-6`

    echo "-1"   > $in
    echo $file >> $in
    echo "T"   >> $in
    echo "F"   >> $in
    woverlap -1 < $in | awk -F= '{print $1,"=", $3}' > $out
    set len = `grep $rfile $out | awk '{if(NR==1) print $4}'`
    set dur   = $len


#...test whether the window start is before of after the times end
    compdate $wst $tend
    if ( $status == 0 ) then
      echo 'Window start > times end: next time'
      goto new_time
    endif

#...test whether the window end is before of after the times start
    compdate $wen $tstart
    if ( $status == 1 ) then
      echo 'Window end < times start: next remote'
      goto new_remote
    endif

#...test whether the remote start is before of after the times end
    compdate $rst $tend
    if ( $status == 0 ) then
      echo 'Remote start > times end: next time'
      goto new_time
    endif

#...test whether the remote end is before of after the times start
    compdate $ren $tstart
    if ( $status == 1 ) then
      echo 'Remote end < times start: next remote'
      goto new_remote
    endif

#...test whether the window start is before of after the remote start
    compdate $wst $rst
    if ( $status == 0 ) then
      set start = `echo $wst`
    else
      set start = `echo $rst`
    endif

#...test whether the window end is before of after the remote end
    compdate $wst $rst
    if ( $status == 0 ) then
      set end = `echo $wen`
    else
      set end = `echo $ren`
    endif


#...handle times start and end
#...test whether the window start is before of after the times start
    compdate $start $tstart
    if ( $status == 1 ) set start = `echo $tstart`

#...test whether the window end is before of after the times end
    compdate $end $tend
    if ( $status == 0 ) set end = `echo $tend`


    echo 'Cascading with '$rfile ' times: '$start' - '$end

# Start processing cycle
#
    if ( ! $do_cas[$ib] ) goto do_rrrmt

      echo "0"         > $in          # print level
      echo "Y"        >> $in          # remote reference
      echo $mode      >> $in          # MT processing
      echo "Y"        >> $in          # STACK
      echo "Y"        >> $in          # JJANAL
      echo $file      >> $in
      echo $rfile     >> $in
      echo "b"        >> $in          # BINARY local  input
      echo "b"        >> $in          # BINARY remote input
      echo $start     >> $in          # start time
      echo $end       >> $in          # end   time
      echo $p_coord   >> $in          # processing coordinate system
      rm -f $site/$file.out7
#

      set window = `echo $file | cut -c4-8`
      if( $mode == 'GDS' || $mode == 'gds') then
        set casfile = $window$rnum'_'$mode.cas_$i
       else
        set casfile = $window$rnum.cas_$i
      endif
      set cascade_list = ( $cascade_list $casfile )

#...remove .out & .cas file if it exists
      rm -f $site/$window$rnum.out >& /dev/null
      rm -f $site/$window$rnum.cas >& /dev/null

      if( $debug == 0 ) then
        tscascade -ask < $in >& $out
      else
        echo 'Input to tscascade.................................'
        cat $in
        echo 'End input..........................................'
        tscascade -ask < $in 
      endif

      mv $site/$window$rnum.cas $site/$casfile
      if ( $debug == 1 ) echo 'New cascade file.... '$site/$casfile

      @ i = ( $i + 1 )

new_time:
  end
new_remote:
 end

end

#...write out cascade files
  rm -f $site/cascade.lst >& /dev/null
  touch $site/cascade.lst
  foreach file ( $cascade_list )
    echo $file >> $site/cascade.lst
  end

#
# restack section
#
  echo "Starting stacking run..."
  echo "0"     > $in
  echo $site  >> $in
  echo "Y"    >> $in
  echo $mode  >> $in 
  if ( $band == '1.000000$' ) then
    echo 0.75    >> $in
  else if ( $band == '0.200000$' ) then
    echo 0.50    >> $in
  else if ( $band == '0.033333$' ) then
    echo 0.25    >> $in
  endif
  echo "0.9999" >> $in
  echo "Y"      >> $in
  echo "JKN"    >> $in
  echo " "      >> $in
  echo "Y"      >> $in
  foreach file ( $cascade_list )
    if ( -e $site/$file ) echo $file >> $in
  end
  echo "stop"     >> $in
  echo "$strike"  >> $in
  rm -f $site/$site.xr $site/$site'.xrg' $site/$site'r.dat' $site/$site.out


  if( $debug == 0 ) then
    tsrestack -ask < $in >& tsrestack_out
  else
    echo 'Input to tsrestack.................................'
    cat $in
    echo 'End input..........................................'
    tsrestack -ask < $in
  endif


  rm $site/*.out

  if( $mode == 'GDS' || $mode == 'gds') then
    mv $site/$site'r.dat $site/$site'_'$ib'r'$mode'.dat'
   else
    mv $site/$site'r.dat' $site/$site'_'$ib'r.dat'
  endif
  rm -f $out $in


new_band:
  @ ib = ( $ib + 1 )
end

goto exit

#
# merge output files together
#
lims_merge $site 
#
#
# plot final data file
#
# mt_plot jfor/$site.dat
#
#
# create .edi file
#
cd jfor
j2edi <<EOD
$site
stop
12
EOD
mv $site.edi ~/edi
echo ' '
echo ' '
echo 'edi file created (in edi directory)'
cd ..
#
# cleanup files
#

rm -f $in $out 
rm -f $site/$site??.out7 

if ( $keep == 0) rm -f $site/*.cas_?

exit (0)
