#!/bin/csh -f

onintr exit

#
# 4.00 Hz, 1.00 Hz, 5.0 second and 30.0 second bands for data processing
# do_bands set to 1 if band is to be reprocessed, 0 if not
# do_cas  set to 1 if tscascade is to be used, 0 if rrrmt in that band
#
set bands = ( '4.000000$' '1.000000$' '0.200000$' '0.033333$' )
set do_bands = (   0        0       1      0 )
set do_cas   = (   0        0       1      0 )
#
# it is recommended that you use tscascade in your highest band
# and rrrmt in the others
#
# this script can be run any number of times in parallel with the
# following caveat:
#   if you do rrrmt processing in a band you must ensure multiple
#   instances of processing do not occur simultaneously for the same 
#   site
#
# Sift through all files in project windows file for remote processing
#
# D.E. Boerner July 1994
# mods         Oct  1995
#
# v4.x mods: agj may 1997
#
# added daily intervals: agj sept 1998
#
if ( $#argv == 0 ) then
  echo 'Automatic .ts processing script. '
  echo ' '
  echo 'Sifts through all files in windows.xxx file for processing '
  echo ' '
  echo '* IGNORES ALL SPLICED DATA except XS and AS files*'
  echo ' '
  echo 'Uses 1S files for  1 sec data' 
  echo '     AS files for  5 sec data'
  echo '     XS files for 30 sec data'
  echo ' '
  echo ' You can keep the intermediate cascade files by setting keep=1'
  echo ' '
  echo 'USAGE: lims_remote_times_all <site_number> <MT or GDS> <strike> <daily start hour> <daily end hour> <keep/debug>'
  echo 'i.e. : lims_remote_times_all 001 mt 0 06 18 keep/debug' 
  echo ' '
  exit(0)
else
  set num = $argv[1]
  shift
endif

if ( $num < 10 ) then
  set num = `echo $num | cut -c3-3 `
  set num = '00'$num
else if ( $num < 100 && $num > 9 ) then
  set num = `echo $num | cut -c2-3 `
  set num = '0'$num
endif

if ($#argv == 0) then
  echo -n "MT or GDS ? "
  set mode = ($<) 
else
  set mode = $argv[1]
  shift
endif

if ($#argv == 0) then
  echo -n "Strike angle ? "
  set p_coord = ($<) 
 else
  set p_coord = $argv[1]
  shift
endif
set strike = $p_coord

if ($#argv == 0) then
  echo -n "Start hour (hh) ? "
  set start_hour = ($<) 
 else
  set start_hour = $argv[1]
  shift
endif
set start_hour = `echo $start_hour | cut -c1-2`
if( $start_hour < 10 ) then
  set start_hour = '0'$start_hour
endif


if ($#argv == 0) then
  echo -n "End hour (hh) ? "
  set end_hour = ($<) 
 else
  set end_hour = $argv[1]
  shift
endif
set end_hour = `echo $end_hour | cut -c1-2`
if( $end_hour < 10 ) then
  set end_hour = '0'$end_hour
endif

set dur  = `echo $end_hour $start_hour | awk '{print $1 - $2}'`
if( $dur < 0 ) set dur  = `echo $dur  | awk '{print 24 + $1}'`
if( $dur < 10 ) then
  set dur = '0'$dur
endif
set dur = $dur'00'

if ($#argv != 0) then
  set param = $argv[1]
  if( $param == keep ) then
    set keep  = 1
    set debug = 0
  else if( $param == debug ) then
    set keep  = 0
    set debug = 1
  else
    set keep  = 0
    set debug = 0
  endif
else
  set keep  = 0
  set debug = 0
endif


set ptime = `date`
set ptime = `echo $ptime | cut -c4-16,20-28`

set ib = 1
set srvy = `echo $cwd | awk -F/ '{print $NF}'` 
set wf   = 'windows.'$srvy
set site   = $srvy$num
set log    = $site/auto_process.log
set in     = lims_remote_file_iput_$$
set out    = lims_remote_file_oput_$$
rm -f $in $out 


grep $num $srvy'site.flt' >& /dev/null
if ( $status != 0 ) then
  echo 'Site '$srvy$num' is not in the flat file '$srvy'site.flt' 
  echo 'You must copy this file from the PC to the SPARC'
  exit(0)
endif
echo ' ' 
#
if ( ! ( -e $log ) ) touch $log

foreach band ( $bands ) 
  set bd = `echo $band | cut -c1-8`
  if ( ! $do_bands[$ib] ) then
    if( $debug == 1 ) echo 'Skipping band: ' $bd '  Hz'
    goto new_band
  else
    if( $debug == 1 ) echo 'Doing band   : ' $bd '  Hz'
  endif
  grep $site $wf | grep $band  >& /dev/null
  if ( $status ) goto new_band
  echo '----------------------NEW BAND: '$bd' Hz -------------------------'
#
#  set files = `grep $site $wf | grep $band | grep -v as | awk '{print $1}'`
#
  if ( $band == '1.000000$' ) then
    set files = `grep $site'1s' $wf | awk '{print $1}'`
  else if ( $band == '0.200000$' ) then
    set files = `grep $site'as' $wf | awk '{print $1}'`
   else
    set files = `grep $site $wf | grep $band | grep -v $site'1s' | awk '{print $1}'`
  endif

  if ( $band == '1.000000$' ) then
    set remote_list = `grep '1s ' $wf | grep -v $site | grep -v '#' | awk '{print $1}'`
  else if ( $band == '0.200000$' ) then
    set remote_list = `grep 'as ' $wf | grep -v $site | grep -v '#' | awk '{print $1}'`
  else if ( $band == '0.033333$' ) then
    set remote_list = `grep 'xs ' $wf | grep -v $site | grep -v '#' | awk '{print $1}'`
  endif
  set cascade_list = ' '

  echo 'Remote list of files -->'
  echo $remote_list

  foreach file ( $files )
    foreach rfile ( $remote_list )
    echo 'Forming cascade decimates with file '$rfile
#
# form cascade decimates with all available other files (excluding local)
#

    set rnum  = `echo $rfile | cut -c4-6`
#
    echo "-1"   > $in
    echo $file >> $in
    echo "T"   >> $in
    echo "F"   >> $in
    woverlap -1 < $in | awk -F= '{print $1,"=", $3}' > $out
    set len = `grep $rfile $out | awk '{if(NR==1) print $4}'`

#...get the start and end times
    set wst = `grep $file  $wf | awk '{print $4}' | awk -F- '{print $1}'`
    set wen = `grep $file  $wf | awk '{print $4}' | awk -F- '{print $2}'`
    set rst = `grep $rfile $wf | awk '{print $4}' | awk -F- '{print $1}'`
    set ren = `grep $rfile $wf | awk '{print $4}' | awk -F- '{print $2}'`

    if ( $rst > $wen ) then
      echo 'Remote start > window end'
#      echo  $rst         > $wen
      goto new_station
    endif
    if ( $ren < $wst ) then
      echo 'Remote end < window start'
#      echo  $ren       < $wst
      goto new_station
    endif

    if ( $wst < $rst ) set wst = $rst
    if ( $wen > $ren ) set wen = $ren

    set overlap_start = $wst
    set overlap_end   = $wen
    set overlap_dur   = $len

    set day_start    = `echo $overlap_start | cut -c1-6`
    set day_end      = `echo $overlap_end   | cut -c1-6`

    echo $day_start     >  $in
    echo $day_end       >> $in
    days < $in > $out
    set days = `cat $out`
    set lastday = `echo $days | awk '{print $NF}'`

    if( $debug == 1 ) echo 'Overlap start, end and duration >', $overlap_start, $overlap_end, $overlap_dur

#...loop around with start_hour -> end_hour for each day

    set iday = 1
    set nday = 2
    foreach day ( $days )


      set start = $day$start_hour'0000'
      if( $start_hour < $end_hour ) then
        set end   = $day$end_hour'0000'
      else
        if( $day == $lastday ) then
          goto new_station
        endif
        set next_day = $days[$nday]
        set end   = $next_day$end_hour'0000'
      endif

      if( $debug == 1 ) echo 'Start and End times =' $start'-'$end

      if( $start < $overlap_start ) then
        echo 'Start ('$start') < Overlap_start ('$overlap_start') : Day skipped'
        goto next_day
      endif

      if( $end > $overlap_end ) then
        echo 'End ('$end') > Overlap_end ('$overlap_end'): Next station'
        goto new_station
      endif

#
# Start processing cycle
#
    if ( ! $do_cas[$ib] ) goto do_rrrmt
    set i = 1
    while ( $i <= $#start ) 
      echo "0"         > $in          # print level
      echo "Y"        >> $in          # remote reference
      echo $mode      >> $in          # MT processing
      echo "Y"        >> $in          # STACK
      echo "Y"        >> $in          # JJANAL
      echo $file      >> $in
      echo $rfile     >> $in
      echo "b"        >> $in          # BINARY local  input
      echo "b"        >> $in          # BINARY remote input
      echo $start[$i] >> $in          # start time
      echo $end[$i]   >> $in          # end   time
      echo $p_coord   >> $in          # processing coordinate system
      rm -f $site/$file.out7
#

      set window = `echo $file | cut -c4-8`
      if( $mode == 'GDS' || $mode == 'gds') then
        set casfile = $window$rnum'_'$mode.cas_$i
       else
        set casfile = $window$rnum'.cas_'$day'_'$i
      endif
      set cascade_list = ( $cascade_list $casfile )
#
# check $log file to see if this particular combination of data has been
# previously cascaded. If so, check the cascade file exists and restack
#
      set lab = $file' & '$rfile' (RR)'
      grep $file $log | grep $rfile > $out
      if( $status == 1 || ! ( -e $site/$casfile ) ) then
        grep -v '''$lab''' $log > $out 
        mv $out $log
        echo $lab': '$bd' Hz: Overlap='$dur[$i]':' $ptime >> $log
        echo $lab': Start & End =' $start-$end'  Overlap='$dur[$i]

#...remove .out file if it exists
        rm -f $site/$window$rnum.out >& /dev/null

       if( $debug == 0 ) then
         tscascade -ask < $in >& $out
       else
         echo 'Input to tscascade.................................'
         cat $in
         echo 'End input..........................................'
         tscascade -ask < $in 
       endif


        mv $site/$window$rnum.cas $site/$casfile

      else
        echo $lab': Overlap='$dur[$i]': ALREADY DONE' 
      endif
      @ i = ( $i + 1 )
    end
next_day:
    @ iday = ( $iday + 1 )
    @ nday = ( $iday + 1 )
    end
new_station:
  end

  end

#...write out cascade files
  rm -f $site/cascade.lst >& /dev/null
  touch $site/cascade.lst
  foreach file ( $cascade_list )
    echo $file >> $site/cascade.lst
  end

#
# restack section
#
  echo "Starting stacking run..."
  echo "0"     > $in
  echo $site  >> $in
  echo "Y"    >> $in
  echo $mode  >> $in 
  if ( $band == '1.000000$' ) then
    echo 0.50    >> $in
  else if ( $band == '0.200000$' ) then
    echo 0.50    >> $in
  else if ( $band == '0.033333$' ) then
    echo 0.25    >> $in
  endif
  echo "0.9999" >> $in
  echo "Y"      >> $in
  echo "JKN"    >> $in
  echo " "      >> $in
  echo "Y"      >> $in
  foreach file ( $cascade_list )
    if ( -e $site/$file ) echo $file >> $in
  end
  echo "stop"     >> $in
  echo "$strike"  >> $in
  rm -f $site/$site.xr $site/$site'.xrg' $site/$site'r.dat' $site/$site.out

#  echo 'Input file to tsrestack:...'
#  cat $in

  if( $debug == 0 ) then
    tsrestack -ask < $in >& tsrestack_out
  else
    echo 'Input to tsrestack.................................'
    cat $in
    echo 'End input..........................................'
    tsrestack -ask < $in
  endif


#  mv $site/*cas* /em2/jones
  rm $site/*.out7

#  mv $out $site.restack.out.$window
  if( $mode == 'GDS' || $mode == 'gds') then
    mv $site/$site'r.dat $site/$site'_'$ib'r'$mode'.dat'
   else
    mv $site/$site'r.dat' $site/$site'_'$ib'r_'$start_hour'-'$end_hour'.dat'
  endif
  rm -f $out $in
  goto new_band

do_rrrmt:
  set i = 1
  set lab = $file' & '$rfile' (RRRMT)'
  echo $lab': '$bd' Hz: Overlap='$dur[$i]':' $ptime >> $log
  nice lims_rrrmt_sno $file -r $rfile -s $strike -d $decl -start $start[$i] -dur $dur[$i] -process_in_strike_dir
#
# fix the header & name of the chave .dat file; this is ugly but it works
#
  if ( -e $site/"$site"c.dat ) then
    cd $site
    set file = `ls *.1mp`
    cd ..
    if ( $#file != 0 ) then
      mphead $file[1] >& head
      set lt = `grep Lat head | cut -c1-27 | awk '{print  $2 + $3/60 + $4/3600 }'`
      set ln = `grep Lon head | cut -c1-27 | awk '{print -$2 - $3/60 - $4/3600 }'`
    endif
    rm -f head

    echo '# MT and GDS estimates from RRRMT' > $site/az.$$ 
    echo '# '`date`                         >> $site/az.$$
    echo '# '                               >> $site/az.$$
    echo '>AZIMUTH    =    ' $strike'.0'    >> $site/az.$$
    echo '>LATITUDE   =    ' $lt            >> $site/az.$$
    echo '>LONGITUDE  =    ' $ln            >> $site/az.$$
    echo '>ELEVATION  =    0.0'             >> $site/az.$$
    cat $site/az.$$  $site/"$site"c.dat > $site/"$site"_"$ib"c.dat
    rm -f  $site/az.$$ 
  endif

new_band:
  @ ib = ( $ib + 1 )
end
#
# merge output files together
#
lims_merge $site 
#
#
# plot final data file
#
# mt_plot jfor/$site.dat
#
#
# create .edi file
#
cd jfor
j2edi <<EOD
$site
stop
12
EOD
mv $site.edi ~/edi
echo ' '
echo ' '
echo 'edi file created (in edi directory)'
cd ..
#
# cleanup files
#

rm -f $in $out 
rm -f $site/$site??.out7 

if ( $keep == 0) rm -f $site/*.cas* $site/cascade.lst

exit (0)
