#!/bin/csh  -f
#
#  lims_tscascade
#
#  runs tscascade
#

onintr exit

if( -e .lims.verbose) then
  echo 'Entered lims_tscascade script...'
  if ( $#argv != 0 ) echo 'Argument 1:' $1
  if ( $#argv == 2 ) echo 'Argument 2:' $2
endif

set old=$cwd

set scripts_dir = `cat .lims.scripts_dir`
set execs_dir   = `cat .lims.execs_dir`

if ( $#argv != 0 ) then
  set ans = $1
  if ( $ans == LOCAL ) then
    set local = y
  else
    set local = n
  endif
endif



if ( $#argv == 2 ) then
  set back = y
else
  set back = n
endif


start:

if ( $back == y ) then
  echo -n 'Run tscascade locally (y/n) [default: y] ? >'
  set ans = $<
  if ( $ans == "" || $ans == y || $ans == Y ) then
    set machine = local
  else
    echo -n 'Give machine name ? >'
    set machine = $<
  endif
endif

set root = ` `

set stn      = `cat .lims.stn`
set dbase    = `cat .lims.dbase`
set stn_dir  = `cat .lims.stn_dir`
set lims_dir = `cat .lims.lims_dir`
set quick    = `cat .lims.quick`

#...check process.cfg file for occurance of keywords

cd $lims_dir$dbase

if !( -e process.cfg ) then
  echo 'This script cannot be run until the process.cfg file has been created'
  goto exit
endif

grep '#NUMSTK:' process.cfg  >  .lims.process.cfg
grep '#VARTOL:' process.cfg  >> .lims.process.cfg
grep '#OWRITE:' process.cfg  >> .lims.process.cfg
grep '#PROTYP:' process.cfg  >> .lims.process.cfg
grep '#DECLIN:' process.cfg  >> .lims.process.cfg
grep '#ROTANG:' process.cfg  >> .lims.process.cfg
grep '#STACK :' process.cfg  >> .lims.process.cfg
grep '#JJANAL:' process.cfg  >> .lims.process.cfg
grep '#MAXTYP:' process.cfg  >> .lims.process.cfg
grep '#NTYPE :' process.cfg  >> .lims.process.cfg
grep '#WEIGHT:' process.cfg  >> .lims.process.cfg

set nl = `wc -l .lims.process.cfg  | awk '{print $1}'`

if ( $nl > 0 ) then
  echo ' '
  echo 'This script cannot be run as the following variables are commented out'
  echo 'in the process.cfg file -->'
  echo ' '
  cat .lims.process.cfg
  echo ' '
  echo 'Edit process.cfg file and re-run'
  goto exit
endif



cd $stn_dir

window:
  echo  ' ' 
  echo  '  WINDOWS FOUND IN: '$stn_dir
  echo  ' '
  ls *.ts

  set nts = `ls -1  *.ts | wc | awk '{print $1}'`

  if ( $nts > 1 ) then
    echo  ' ' 
    echo ' Enter Run number & Window (no extension) (e.g. a1) '
window_select:
    echo ' '
    echo -n 'Window? ===> '$root
    set window=$<
    set window=$stn$window
    set tts=$window'.ts'
    if ( -e $tts ) then
      echo 'Window selected =>'$window'.ts'
    else
      echo 'Window not found; re-select'
      goto window_select
    endif

  else
    set window = `ls -1  *.ts | awk -F. '{print $1}'`

  endif

goto remote   #---assumed analysis type set in process.cfg

analysis:
  echo  ' ' 
  echo '>>>>Enter analysis type [MT(m), GDS(g) or HSG(h)] >'
analysis_select:
  echo ' '
  echo -n 'Analysis type [default: m] ? ===>'$root
  set anal=$<

  if( $anal == "" ) then
    set anal = m
  endif

  if( $anal == "m" ) then
#    echo ' >MT analysis chosen'
  else if( $anal == "g" ) then
#    echo ' >GDS analysis chosen'
  else if( $anal == "h" ) then
#    echo ' >HSG analysis chosen'
  else
    echo '------erroneous input; choose either m, g or h'
    goto analysis
  endif


remote:
if ( $local == y ) then
  set rrwindow = 'local'
  goto times
endif
  
  cd $lims_dir$dbase
  echo ' '
  set line = `grep $window windows.$dbase`
  echo $line > .lims_local
  set stime = `awk -F-  '{print $1}' .lims_local | awk '{print $4}'`
  set etime = `awk -F-  '{print $2}' .lims_local | awk '{print $1}'`

  if( -e .lims.verbose) then
    echo 'Start and end times for window: stime: '$stime  '   etime: '$etime
  endif

  echo $execs_dir'/wlength -1 > .lims.duration  <<END'  > .lims_wlength
  echo '-1'                                       >> .lims_wlength   #iprint
  echo $stime                                     >> .lims_wlength   #start time
  echo $etime                                     >> .lims_wlength   #end time
  chmod +x .lims_wlength
  .lims_wlength
  set duration = `cat .lims.duration`

  echo 'Window has interval: ' $stime'-'$etime'   duration: '$duration
  echo ' '
  echo 'Determining possible remote-reference windows'
  echo $execs_dir'/woverlap -1 <<END >.lims.remote'         >  .lims_woverlap
  echo '-1'                                                 >> .lims_woverlap
  echo $window                                              >> .lims_woverlap
  echo 'T'                                                  >> .lims_woverlap
  echo 'F'                                                  >> .lims_woverlap
  echo 'END'                                                >> .lims_woverlap
  chmod +x .lims_woverlap
  .lims_woverlap
  sort +4 .lims.remote

  echo  ' ' 
  echo ' Enter Remote Reference station number (last 3 characters)'
remote_select:
  echo ' '
  echo -n 'Remote? ===> '$root
  set remote=$<
  set remote=$dbase$remote
  if( -e $remote ) then
    echo 'Remote selected =>'$remote
  else
    echo 'Remote not found; re-select'
    goto remote_select
  endif

  grep $remote .lims.remote > .lims.rlist

  if !( -e .lims.rlist ) then
    echo 'Incorrect station listed! Choose again:'
    cat .lims.remote
    goto remote_select
  endif

  set nstat=`wc -l .lims.rlist | awk '{print $1}'`

  if ( $nstat == 1 ) then
    set rrwindow = `grep $remote .lims.remote | awk -F: '{print $1}'`
  else
    echo 'More than one window for this station:'
    cat .lims.rlist
    echo ' '
    echo -n 'Remote window (last two characters, e.g. a1) ? ===> '$root
    set rrwindow=$<
    set rrwindow = `grep $rrwindow .lims.rlist | awk -F: '{print $1}'`
  endif

  echo 'Remote window selected =>'$rrwindow'.ts'

times:
  if( $local == y ) then
    cd $lims_dir$dbase
    set line = `grep $window windows.$dbase`
    echo $line > .lims_local
    set stime = `awk -F-  '{print $1}' .lims_local | awk '{print $4}'`
    set etime = `awk -F-  '{print $2}' .lims_local`

    echo $execs_dir'/windowlength -1 > .lims.duration  <<END'  > .lims_wlength
    echo '-1'                                       >> .lims_wlength   #iprint
    echo $stime                                 >> .lims_wlength   #start time
    echo $etime                                 >> .lims_wlength   #end time
    chmod +x .lims_wlength
    .lims_wlength
    set duration = `cat .lims.duration`
  else
    set line = `grep $rrwindow .lims.rlist`
    echo $line > .lims.remote
    set stime = `awk -F=  '{print $2}' .lims.remote | awk -F- '{print $1}'`
    set etime = `awk -F=  '{print $2}' .lims.remote | awk -F- '{print $2}' | awk '{print $1}'`
    set duration = `awk -F=  '{print $3}' .lims.remote`
  endif

  echo ' '
stime_select:
  if ( $quick != y ) then
    echo 'Give start time (default  >'$stime')'
    echo -n 'Start time?            ===>'$root
    set ans=$<
    if !( $ans == "" ) then
      set stime = $ans
    endif
  endif

etime_select:
  if ( $quick != y ) then
    echo 'Give end   time (default  >'$etime')'
    echo -n 'End   time?            ===>'$root
    set ans=$<
    if !( $ans == "" ) then
      set etime = $ans
    endif
  endif

goto tscascade   #---skip asking for azimuth: assumed to be in process.cfg

azimuth:
  echo ' '
  set azim = 0
  echo     ' Give azimuth: default >'$azim
  echo -n  'Azimuth?            ===>'$root
  set ans=$<
  if !( $ans == "" ) then
    set azim = $ans
  endif



tscascade:

  cd $lims_dir$dbase
  echo 'tscascade: window='$window' rrwindow='$rrwindow' stime='$stime' duration='$duration >> lims.log

if( $back == y ) then

  echo 'cd ' $lims_dir$dbase            > .lims_tscascade.$$
  if ( $machine == eos ) then  
    echo '/local/mt/conbin/tscascade -1 > .lims.tscascade.out <<END'  >> .lims_tscascade.$$
  else
    echo $execs_dir'/tscascade -1  > .lims.tscascade.out <<END'    >> .lims_tscascade.$$
  endif
else
  echo $execs_dir'/tscascade -1  <<END'    > .lims_tscascade.$$
endif
  echo '-1'                     >> .lims_tscascade.$$   #iprint level
if( $local == y ) then
  echo 'n'                      >> .lims_tscascade.$$   #local processing
else
  echo 'y'                      >> .lims_tscascade.$$   #RR processing
endif
  echo $window                  >> .lims_tscascade.$$   #window
if( $local == n ) then
  echo $rrwindow                >> .lims_tscascade.$$   #rr window
endif
  echo 'u'                      >> .lims_tscascade.$$   #unformatted local 
if( $local == n ) then
  echo 'u'                      >> .lims_tscascade.$$   #unformatted remote
endif
  echo $stime                   >> .lims_tscascade.$$   #start time
  echo $etime                   >> .lims_tscascade.$$   #end time
  echo ' '                      >> .lims_tscascade.$$   #length correct
  echo 'END'                    >> .lims_tscascade.$$

  chmod +x .lims_tscascade.$$
  if( $back == y ) then
    if ( $machine == local ) then
      echo 'Backgrounding tscascade job locally...'
      .lims_tscascade.$$ &
    else
      echo 'Backgrounding tscascade job on machine '$machine'...'
      rsh $machine $lims_dir$dbase'/.lims_tscascade.'$$  &
    endif
  else
    .lims_tscascade.$$
  endif







exit:
  cd $old
