#!/bin/csh  -f
#
#  lims_tsrestack
#
#  restacks output from limscas7
#

onintr exit

set old=$cwd

set scripts_dir = `cat .lims.scripts_dir`
set execs_dir   = `cat .lims.execs_dir`


if ( $#argv == 1 ) then
  set back = y
else
  set back = n
endif


start:
set root = ` `

set stn      = `cat .lims.stn`
set dbase    = `cat .lims.dbase`
set stn_dir  = `cat .lims.stn_dir`
set lims_dir = `cat .lims.lims_dir`
set stnnum   = `sed y/$dbase/'   '/ .lims.stn | awk '{print$0}'`

#...check process.cfg file for occurance of keywords

cd $lims_dir$dbase

if !( -e process.cfg ) then
  echo 'This script cannot be run until the process.cfg file has been created'
  goto exit
endif

grep '#OWRITE:' process.cfg  >> .lims.process.cfg
grep '#PROTYP:' process.cfg  >> .lims.process.cfg
grep '#ROTANG:' process.cfg  >> .lims.process.cfg
grep '#JJANAL:' process.cfg  >> .lims.process.cfg
grep '#MAXTYP:' process.cfg  >> .lims.process.cfg
grep '#NTYPE :' process.cfg  >> .lims.process.cfg
grep '#WEIGHT:' process.cfg  >> .lims.process.cfg
grep '#COHMIN:' process.cfg  >> .lims.process.cfg
grep '#COHMAX:' process.cfg  >> .lims.process.cfg

set nl = `wc -l .lims.process.cfg  | awk '{print $1}'`

if ( $nl > 0 ) then
  echo ' '
  echo 'This script cannot be run as the following variables are commented out'
  echo 'in the process.cfg file -->'
  echo ' '
  cat .lims.process.cfg
  echo ' '
  echo 'Edit process.cfg file and re-run'
  goto exit
endif


cd $stn_dir

window:
  echo  ' ' 
  echo  '  CASCADE FILES FOUND IN: '$stn_dir
  echo  ' '
  ls *.cas
  echo  ' '

  set nl   = `ls -1 *.cas | grep    $stnnum..$stnnum | wc -l | awk '{print $1}'`
  set nr   = `ls -1 *.cas | grep -v $stnnum..$stnnum | wc -l | awk '{print $1}'`
  set ncas = `echo $nl $nr | awk '{print $1+$2}'`
#  set ncas = `ls -1 *.cas | wc -l | awk '{print $1}'`

  ls -1 *.cas | grep    $stnnum..$stnnum > .lims.l
  ls -1 *.cas | grep -v $stnnum..$stnnum > .lims.r


  if ( $ncas == 0 ) then
    echo ' '
    echo 'NO CASCADE FILES TO RESTACK --- choose "l" or "7"'
    echo ' '
    goto exit
   endif


  if ( $nl != 0 && $nr == 0 ) then
    set rr  = l
    set qrr = n
  else if ( $nl == 0 && $nr != 0 ) then
    set rr  = r
    set qrr = y
  endif

  if ( $nl != 0 && $nr != 0 ) then
    echo 'Both local and remote cascade files found ...'
request_remote:
    echo -n 'Local or remote reference processing (l/r) ? =>'$root
    set rr=$<
    if ( $rr == r ) then
      set qrr = y
    else if (  $rr == l ) then
      set qrr = n
    else
      echo 'Incorrect response; please answer l or r ...'
      goto request_remote
    endif
  endif

  if ( ( $rr == l && $nl > 1 ) || ( $rr == r && $nr > 1 ) ) then
    echo ' '
    echo 'Multiple .cas files found....'
    cat .lims.$rr
request_all:
    echo -n 'Do you want ALL cas files to be averaged (y/n) ? =>'$root
    set all=$<
    if ( $all != n && $all != y ) then
      echo '...answer y or n please...'
      goto request_all
    endif

    if ( $all == y ) then

      cat .lims.$rr  > ../.lims.casfiles
      echo 'stop'   >> ../.lims.casfiles

    else
      rm -f ../.lims.casfiles > /dev/null
      touch ../.lims.casfiles

      echo ' '
request_cas:
      echo -n 'Give cas filename (without extension: <CR> to end input) >'
      set casfile=$<
      if ( $casfile == "" || $casfile == 'stop' ) then
        echo 'stop' >> ../.lims.casfiles
        goto end_request_cas
      endif
      if !( -e $casfile'.cas' ) then
        echo 'Cascade file does not exist: '$casfile
      else
        echo $casfile'.cas' >> ../.lims.casfiles
      endif
      goto request_cas

    endif

  else

    set all = y
    cat .lims.$rr  > ../.lims.casfiles
    echo 'stop'   >> ../.lims.casfiles

  endif

end_request_cas:

goto tsrestack          #---skip asking lots of stuff; assumed in process.cfg

analysis:
  echo  ' ' 
  echo '>>>>Enter analysis type [MT(m), GDS(g) or HSG(h)] >'
analysis_select:
  echo ' '
  echo -n 'Analysis type [default: m] ? ===>'$root
  set anal=$<

  if( $anal == "" ) then
    set anal = m
  endif

  if( $anal == "m" ) then
#    echo ' >MT analysis chosen'
  else if( $anal == "g" ) then
#    echo ' >GDS analysis chosen'
  else if( $anal == "h" ) then
#    echo ' >HSG analysis chosen'
  else
    echo '------erroneous input; choose either m, g or h'
    goto analysis
  endif


ctype:
  echo ' '
  echo 'Enter COH (maximise coherence), or VAR or JKN (both minimise variance)'
  echo '*********************choose JKN*********************'
ctype_select:  
  echo -n 'Type  (default JKN) ? ==>'$root
  set ctype=$<
  if ( $ctype == "" ) set ctype = JKN

ntype:
  if( $ctype == COH || $ctype == coh ) then
    echo ' '
    echo 'Enter ntype (1-7)'
    goto ntype_select
  else if ( $ctype == VAR || $ctype == var ) then
    echo ' '
    echo 'Enter ntype (1-4)'
    goto ntype_select
  else if ( $ctype == JKN || $ctype == jkn ) then
    echo ' '
    echo 'NTYPE MUST BE selected now'
    echo 'Enter ntype (1-3)'
    goto ntype_select
  else
    echo 'Invalid type entered =>'$ctype'   re-enter'
    goto ctype_select
  endif

ntype_select:
  echo -n 'Ntype (O,o for options) ==>'$root
  set ntype=$<
  if ( $ntype == "" ) set ntype = 1
  if ( $ntype == "O" || $ntype == "o" ) then
    if ( $ctype == COH || $ctype == coh ) then
      echo 'type=COH   '
      echo 'ntype=1   maximize  Mcoh(ex,hx,hy)*Mcoh(ey,hx,hy)'
      echo '      2             Mcoh(ex,hx,hy)'
      echo '      3             Mcoh(ey,hx,hy)'
      echo '      4             Pcoh(ex,hy,hx)*Pcoh(ey,hx,hy)'
      echo '      5             Pcoh(ex,hy,hx)'
      echo '      6             Pcoh(ey,hx,hy)'
      echo '      7             Mcoh(rx,hx,hy)*Mcoh(ry,hx,hy)'
    else if ( $ctype == VAR || $ctype == var ) then
      echo 'type=VAR   '
      echo 'ntype=1   minimize  Sd31*Sd32*Sd41*Sd42'
      echo '      2             Sd31*Sd32'
      echo '      3             Sd41*Sd42'
      echo '      4             Sd32*Sd41'
    else if ( $ctype == JKN || $ctype == jkn ) then
      echo 'type=JKN   '
      echo 'ntype=1   minimize  Var(Zxy)*Var(Zyx)'
      echo '      2             Var(Zxy)'
      echo '      3             Var(Zyx)'
    endif
    goto ntype_select
  endif

weights:
  echo ' '
  echo 'Do you want to form a weighted stack (or unweighted)'
  echo -n  'Weights (y/n) [default: y] >'$root
  set wgts=$<
  if( $wgts == "" ) set wgts=y

cohmin:
  echo ' '
  echo -n 'Choose coherence minimum for acceptance (default 0.9) >'$root
  set cohmin=$<
  if( $cohmin == "" ) set cohmin=0.9


angle:
  echo ' '
  echo 'Enter angle to rotate spectra to (default 0.0)'
  echo ' '
  echo -n 'Angle? ==>'$root
  set angle=$<
  if( $angle == "" ) set angle=0.0

tsrestack:

  cd $lims_dir$dbase
#  echo 'tsrestack: rr='$rr' ctype='$ctype' ntype='$ntype' wgts='$wgts' cohmin='$cohmin' angle='$angle >> lims.log
  echo 'tsrestack: rr='$rr >> lims.log

if( $back == y ) then
  echo $execs_dir'/tsrestack  > .lims.tsrestack.out  <<END'   > .lims_tsrestack
else
  echo $execs_dir'/tsrestack    <<END'      > .lims_tsrestack
endif
  echo '-1'                       >> .lims_tsrestack  #print level
  echo $stn                       >> .lims_tsrestack  #station
  echo $qrr                       >> .lims_tsrestack  #RR processing
#  echo $anal                     >> .lims_tsrestack  #analysis type
#  echo $cohmin                   >> .lims_tsrestack  #min coherence
#  echo '0.99'                    >> .lims_tsrestack  #max coherence
#  echo 'y'                       >> .lims_tsrestack  #JJ anal
#  echo $ctype                    >> .lims_tsrestack  #maximise type
#  echo $ntype                    >> .lims_tsrestack  #NTYPE =7
#  if(  $ctype == COH || $ctype == coh ) echo '1.0'  >> .lims_tsrestack  #max coherence
#  echo $wgts                     >> .lims_tsrestack  #stack with weights
  cat .lims.casfiles             >> .lims_tsrestack  #.cas7 files chosen
#  echo $angle                    >> .lims_tsrestack  #angle to rotate spectra to
  echo 'END'                     >> .lims_tsrestack 
 
  chmod +x .lims_tsrestack
  if( $back == y ) then
    echo ' Backgrounding lims_tsrestack script file'
    .lims_tsrestack &
  else
    .lims_tsrestack
  endif




exit:
  cd $old

