#!/bin/csh  -fll .l*

#
#  lims_magplotd
#
#  plots data from a station
#

onintr exit

set old=$cwd

set scripts_dir = `cat .lims.scripts_dir`
set execs_dir   = `cat .lims.execs_dir`

if ( $#argv != 0 ) then
  set dev = 'sel MPOST;e'
  set hardcopy = y
  if ( $#argv == 2 ) then
    ps -cux | grep magplotd  > .lims.magplotd
    set nl = `wc -l .lims.magplotd  | awk '{print $1}'`
    if ( $nl == 0 ) then
      set back = y
    else
      echo 'Only one magplotd job permitted to be in background'
      goto exit
    endif
  else
    set back = n
  endif
else
  set dev = 'sel MX11;e'
  set hardcopy = n
  set back = n
endif

start:
set root = ` `

set stn      = `cat .lims.stn`
set dbase    = `cat .lims.dbase`
set stn_dir  = `cat .lims.stn_dir`
set lims_dir = `cat .lims.lims_dir`
set quick    = `cat .lims.quick`

cd $stn_dir


window:
  echo  ' ' 
  echo  '  WINDOWS FOUND IN: '$stn_dir
  echo  ' '
  ls *.?ts

  set nts = `ls -1  *.?ts | wc | awk '{print $1}'`

  if ( $nts > 1 ) then
    echo  ' ' 
    echo ' Enter Run number & Window (no extension) (e.g. a1) '
window_select:
    echo ' '
    echo -n 'Window? ===> '$root
    set window=$<
    set window=$stn$window
    set tts=$window'.ts'
    if ( -e $tts ) then
      echo 'Window selected =>'$window'.ts'
    else
      echo 'Window not found; re-select'
      goto window_select
    endif

  else
    set window = `ls -1  *.?ts | awk -F. '{print $1}'`

  endif
  
times:
  cd $lims_dir$dbase
  set line = `grep $window windows.$dbase`
  echo $line > .lims_local
  set stime = `awk -F-  '{print $1}' .lims_local | awk '{print $4}'`
  set etime = `awk -F-  '{print $2}' .lims_local`

  echo $execs_dir'/windowlength -1 > .lims.duration  <<END'  > .lims_limscas7
  echo '-1'                                      >> .lims_limscas7   #iprint
  echo $stime                           >> .lims_limscas7   #start time
  echo $etime                           >> .lims_limscas7   #end time
  chmod +x .lims_limscas7
  .lims_limscas7
  set duration = `cat .lims.duration`


stime_select:
  if ( $quick != y ) then
    echo ' '
    echo ' Give start time of event (default >'$stime')'
    echo -n 'Start time?                     ===>'$root
    set ans=$<
    if !( $ans == "" ) then
      set stime = $ans
    endif
  endif

duration:
  if ( $quick != y ) then
    echo ' '
    echo ' Give duration (ddhhmm): default >'$duration
duration_select:
    echo -n 'Duration?                     ===>'$root
    set ans=$<
    if !( $ans == "" ) then
      set duration = $ans
    endif
  endif

echo 'Press <RETURN> in this command window when finished viewing...'

dec_select:
#  echo ' '
#  echo -n ' Enter decimation factor, or <RETURN> to use default >'
#  set dec = $<
  set dec=' '

cd $lims_dir$dbase

magplotd:
  echo $execs_dir'/xmagp -ps > .lims.magplotd.out  <<END'  >> .lims_magplotd.$$
  echo '-1'                  >> .lims_magplotd.$$   #iprint
  echo $window'.ts'         >> .lims_magplotd.$$   #window to plot
  echo ' '                   >> .lims_magplotd.$$   #number of channels
  echo ' '                   >> .lims_magplotd.$$   #digitizing interval
  echo $stime                >> .lims_magplotd.$$   #start time
  echo $duration             >> .lims_magplotd.$$   #duration
  echo $dec                  >> .lims_magplotd.$$   #decimation factor
  echo ' '                   >> .lims_magplotd.$$   #factors correct
  echo ' '                   >> .lims_magplotd.$$   #no plotting aux channels
  echo ' '                   >> .lims_magplotd.$$   #default Hx factor
  echo ' '                   >> .lims_magplotd.$$   #default Hy factor
  echo ' '                   >> .lims_magplotd.$$   #default Hz factor
  echo ' '                   >> .lims_magplotd.$$   #default Ex factor
  echo ' '                   >> .lims_magplotd.$$   #default Ey factor
  echo ' '                   >> .lims_magplotd.$$   #<CR> to clear plot
  echo ' '                   >> .lims_magplotd.$$   #no replot another
  echo ' '                   >> .lims_magplotd.$$   #no new time frame
  echo 'END'                 >> .lims_magplotd.$$   #END

if ( $hardcopy == y ) then
  echo 'lpr -Pps buplot.ps'  >> .lims_magplotd.$$
endif

  chmod +x .lims_magplotd.$$
  if( $back == y ) then
    echo ' Backgrounding script file'
    /bin/csh .lims_magplotd.$$ &
  else
    /bin/csh .lims_magplotd.$$
  endif
  

if ( $hardcopy != y ) then
  echo ' '
  echo -n 'Hardcopy of the plot y/n ? >'$root
  set ans = $<
  if( $ans == y ) then
    lpr -Pps buplot.ps 
  endif
endif




exit:
cd $old
