c
c     tsfilter
c
c     filters data in a ts format file
c
      implicit none

      include 'ts_include.inc'

      character*1 sep
      character*10 cmdarg
      integer narg, getnarg

      character*6    formi, formo
      character*15 formati, formato
      character*30 fin, fout
      character*80 line
      character*8 tswindow, tswindowout
      character*19 stime, etime
      character*24 date_string
      character*11 fwin
      character*1 FILT
      integer i, ipt, npt, ichan, nchan, instr
      real dat(5), datout(5)
      real*8 x(5,3), y(5,3)
      real*8 n(3), d(3)
      real*8 delta8, cut8
      real cut

      integer tdiff

c-------------------------------------------------------------------------------
c     get date

      call fdate(date_string)

c-------------------------------------------------------------------------------
c     Initialize variables and arrays.

      call dirsep(sep)

c-------------------------------------------------------------------------------
c     get command-line arguments

      narg = getnarg()
      do i = 1, narg
        call getcmdarg( i, cmdarg)
        if( index(cmdarg,'h').ne.0 ) then
          write(*,*)'Permitted options:-'
          write(*,*)'                    -1 -> 3  iprint level'
          stop
        else
          read(cmdarg,'(i2)') iprint
        endif
      enddo

      call iin( 'Give print level (-1/0/1/2/3)',iprint)

c-------------------------------------------------------------------------------
c...write banner

      call banner( 'TSFILTER' )

c-------------------------------------------------------------------------------
c...open window

      tswindow = ' '
      form     = '?'
      call ts_open( ind1, tswindow, fin, .TRUE., .FALSE.)

c...read in info block
      call ts_rdinfo(ind1)
      formi   = form
      formati = format

c-------------------------------------------------------------------------------
c...open widows file and get start and end times

      fwin = 'windows.'//tswindow(1:3)
      goto 405
  400 call tin (' Give the project windows file', fwin )
  405 open ( ind2, file=fwin, status='old', form='formatted', err=410 )
      go to 420
  410 write (*,'(a)') 'tsfilter: Error opening WINDOWS file:'//
     &                '  fwin = ', fwin
      goto 400
  420 continue
 
      call getwindow( ind2, tswindow, instr, nchan, stime, etime )
      write(*,*)'Times from windows file: stime = ', stime
      write(*,*)'                         etime = ', etime

c...derive length of series in points
c   first convert delta to samples/sec
      if( tunits.eq.'Hz' ) then
        continue
      else if( tunits.eq.'s' ) then
        delta = 1. / delta
      else
        write(*,*)'Time units not recognised >', tunits
        write(*,*)'only "s" or "Hz" permitted'
        stop
      endif
      npt = tdiff( stime, etime ) * delta
      write(*,*)'No. of points in series =', npt

c-------------------------------------------------------------------------------
c...get filter info

      FILT = 'L'
      call tin( 'Give filter type: L=low pass, H=high pass', FILT )

      cut = delta/10.
      call rin('Give -3dB frequency', cut )

c...compute filter coefficients
      delta8 = delta
      cut8   = cut
      call fcoef( FILT, delta8, cut8, n, d )
      write(*,*)'Recursion filter coefficients ==>'
      write(*,*) 'Numerator  : ', (n(i),i=1,3)
      write(*,*) 'Denominator: ', (d(i),i=1,3)

c-------------------------------------------------------------------------------
c...open output file - BINARY only !!!

c...set form and format to BINARY and UNFORMATTED
      formo   = 'BINARY'
      formato = 'UNFORMATTED'
      form   = formo
      format = formato

      tswindowout = 'fx'
      call tin( 'Give 2-char output window', tswindowout )
      tswindowout = tswindow(1:6)//tswindowout(1:2)
      write(*,'(a)') 'Opening output file: '// tswindowout
      call ts_open( outd1, tswindowout, fout, .FALSE., .FALSE. )

c...write out comment block
      line = '# tsfilter: '//ver
      write(outd1) line
      line = '# date: '//date_string
      write(outd1) line
      line = '#'
      write(outd1) line
      line = '# 2-pole Butterworth filter applied to all series'
      write(outd1) line
      line = '# Type L = low pass'
      write(outd1) line
      line = '# Type H = high pass'
      write(outd1) line
      line = '#'
      write(outd1) line
      line = '# Filter type   : '//FILT
      write(outd1) line
      line = '# -3dB frequency:'
      write(line(20:),*) cut
      write(outd1) line
      line = '#'
      write(outd1) line
c...copy comment block from input file
      rewind( ind1 )
      call ts_copycomm( ind1, formi, outd1, formo )

c...read in then write out info block
      form   = formi
      format = formati
      call ts_rdinfo( ind1 )
      window = tswindowout
      form   = formo
      format = formato
      starttime = stime
      endtime   = etime
      call ts_wrtinfo( outd1 )

c-------------------------------------------------------------------------------
c...read data into loop and perform recursion

c...initialize arrays
      do ichan = 1, nchan
        do i = 1, 3
          x(ichan,i) = 0.d0
          y(ichan,i) = 0.d0
        enddo
      enddo

c...loop over data reading in all points
      do ipt = 1, npt

c...read one sequence of data into dat array
        form   = formi
        format = formati
        call ts_in1( ind1, dat )
        if( dat(1).eq.EOF ) goto 9001

        do ichan = 1, nchan
          if( dat(ichan).ne.misdat ) then
            x(ichan,1) = x(ichan,2)
            x(ichan,2) = x(ichan,3)
            x(ichan,3) = dble(dat(ichan))
            y(ichan,1) = y(ichan,2)
            y(ichan,2) = y(ichan,3)

            y(ichan,3) = (x(ichan,3)*n(1) + x(ichan,2)*n(2) 
     &                  + x(ichan,1)*n(3) ) -
     &                ( y(ichan,2)*d(2) + y(ichan,1)*d(3) )
            datout(ichan) = sngl(y(ichan,3))
          else
            datout(ichan) = misdat
          endif
        enddo

c...write data out
        form   = formo
        format = formato
        call ts_out1( outd1, datout )
      enddo

      stop

9001  write(*,*) 'EOF found before all data could be read in: ',
     &           ' ipt = ', ipt
      stop

9999  stop
      end



c-------------------------------------------------------------------------------
      subroutine fcoef( FILT, deltin, cut, num, den )
c
c     fcoef
c     routine to calculate the recursion filter coefficients for a
c     3-pt two-pole Butterworth filter, both high pass and low pass
c
c Input
c     FILT    H for high pass filter coeffs, L for low pass filter coeffs
c     deltin  digitizing interval in samples/sec
c     cut     -3dB cutoff frequency in Hz
c Output
c     num     numerator - length 3
c     den     denominator - length 3
c
c      REFERENCE: KULHANEK (1976) PP87-92 & NOTES(JONES)
c
      implicit none

      character*1 FILT
      real*8 deltin, cut, num(3), den(3)

      real*8 w, b, c, weq, beq, ceq, teq, delta, tcut, norm
      integer i

c...convert delta and tcut to seconds
      delta = 1./deltin
      tcut  = 1./cut

      W=TAN(3.14159*cut/deltin)
      B=W*(SQRT(2.0))
      C=W*W

      if( FILT.EQ.'H' ) then
        num(1) = 1./(1.+B+C)
        num(2) = -2.*num(1)
        num(3) = num(1)
C         *DETERMINE 'EQUIVALENT' CUT-OFF PERIOD
        TEQ=( 2.*DELTA*TCUT ) / ( TCUT - 2.*DELTA )
        WEQ=TAN(3.141592*DELTA/TEQ)
        BEQ=WEQ*(SQRT(2.0))
        CEQ=WEQ*WEQ
        den(1) = 1.
        den(2) = -2.*(CEQ-1.)/(1.+BEQ+CEQ)
        den(3) = (1.-BEQ+CEQ)/(1.+BEQ+CEQ)

      elseif( FILT.eq.'L' ) then
        num(1) = 1./(1.+B+C)
        num(2) = 2.*num(1)
        num(3) = num(1)
        den(1) = 1.
        den(2) = 2.*(C-1.)/(1.+B+C)
        den(3) = (1.-B+C)/(1.+B+C)
C         *APPLY NORMALISATION
        NORM=(den(1)+den(2)+den(3))/(num(1)+num(2)+num(3))
        NORM=ABS(NORM)
        do i = 1, 3
          num(i) = num(i)*norm
        enddo

      else
        write(*,*)'Unknown filter type: FILT = ', FILT
        stop

      endif

      return
      end
