c
c     tsfilter
c
c     filters data in a ts format file
c
      implicit none

      include 'ts_include.inc'

      integer MAXP, MAXP4
      parameter ( MAXP  = 524288 )
      parameter ( MAXP4 = 4*MAXP+15 )

      character*1  sep
      character*10 cmdarg
      integer narg, getnarg

      character*6  formi,   formo
      character*15 formati, formato
      character*30 fin, fout
      character*80 line
      character*8 tswindow, tswindowout
      character*19 stime, etime
      character*24 date_string
      character*11 fwin
      integer i, npt, ichan, nchan, instr, nptfft
      real data(5,MAXP), datout(5,MAXP), cr, ci, crl
      real wsave(MAXP4)
      complex cdata(MAXP)

      integer tdiff

c-------------------------------------------------------------------------------
c     get date

      call fdate(date_string)

c-------------------------------------------------------------------------------
c     Initialize variables and arrays.

      call dirsep(sep)

c-------------------------------------------------------------------------------
c     get command-line arguments

      narg = getnarg()
      do i = 1, narg
        call getcmdarg( i, cmdarg)
        if( index(cmdarg,'h').ne.0 ) then
          write(*,*)'Permitted options:-'
          write(*,*)'                    -1 -> 3  iprint level'
          stop
        else
          read(cmdarg,'(i2)') iprint
        endif
      enddo

      call iin( 'Give print level (-1/0/1/2/3)',iprint)

c-------------------------------------------------------------------------------
c...write banner

      call banner( 'TSHILBERT' )

c-------------------------------------------------------------------------------
c...open window

      tswindow = ' '
      form     = '?'
      call ts_open( ind1, tswindow, fin, .TRUE., .FALSE.)

c...read in info block
      call ts_rdinfo(ind1)
      formi   = form
      formati = format

c-------------------------------------------------------------------------------
c...open widows file and get start and end times

      fwin = 'windows.'//tswindow(1:3)
      goto 405
  400 call tin (' Give the project windows file', fwin )
  405 open ( ind2, file=fwin, status='old', form='formatted', err=410 )
      go to 420
  410 write (*,'(a)') 'tsfilter: Error opening WINDOWS file:'//
     &                '  fwin = ', fwin
      goto 400
  420 continue
 
      call getwindow( ind2, tswindow, instr, nchan, stime, etime )
      write(*,*)'Times from windows file: stime = ', stime
      write(*,*)'                         etime = ', etime

c...derive length of series in points
c   first convert delta to samples/sec
      if( tunits.eq.'Hz' ) then
        continue
      else if( tunits.eq.'s' ) then
        delta = 1. / delta
      else
        write(*,*)'Time units not recognised >', tunits
        write(*,*)'only "s" or "Hz" permitted'
        stop
      endif
      npt = tdiff( stime, etime ) * delta
      write(*,*)'No. of points in series =', npt
      if( npt.gt.MAXP ) then
        write(*,*)' npt > MAXP '
        stop
      endif


c-------------------------------------------------------------------------------
c...open output file - BINARY only !!!

c...set form and format to BINARY and UNFORMATTED
      formo   = 'BINARY'
      formato = 'UNFORMATTED'
      form   = formo
      format = formato

      tswindowout = tswindow(1:6)//'hx'
      write(*,'(a)') 'Opening output file: '// tswindowout
      call ts_open( outd1, tswindowout, fout, .FALSE., .FALSE. )

c...write out comment block
      line = '# tshilbert: '//ver
      write(outd1) line
      line = '# date: '//date_string
      write(outd1) line
      line = '#'
      write(outd1) line
      line = '# Hilbert transformed instantaneous amplitude'
      write(outd1) line
      line = '#'
c...copy comment block from input file
      rewind( ind1 )
      call ts_copycomm( ind1, formi, outd1, formo )

c...read in then write out info block
      form   = formi
      format = formati
      rewind(ind1)
      call ts_rdinfo( ind1 )

      window = window(1:6)//'hx'
      form   = formo
      format = formato
      starttime = stime
      endtime   = etime
      nchan=3
      call ts_wrtinfo( outd1 )
      nchan=5

c-------------------------------------------------------------------------------
c...read data into loop and perform recursion

c...convert delta to samples/sec
      if( tunits.eq.'Hz' ) then
        continue
      else if( tunits.eq.'s' ) then
        delta = 1. / delta
      else
        write(*,*)'Time units not recognised >', tunits
        write(*,*)'only "s" or "Hz" permitted'
        stop
      endif

      form   = formi
      format = formati
      call ts_read( ind1, stime, etime, data )
      write(*,*)'All data read in'

c-------------------------------------------------------------------------------
c...perform Hilbert transform

c...find next radix of power of 2 for length of series
      do i = 1, 20
        if( 2**i.lt.npt .and. 2**(i+1).gt.npt ) then
          nptfft = 2**(i+1)
          write(*,*) 'Next highest radix of 2 is:', nptfft
          goto 100
        endif
      enddo
      write(*,*)'Impossibly large number !' 
      stop
100   continue

c...initialize fft routines
      call cffti( nptfft, wsave )

      do ichan = 1, nchan
        write(*,*)'Performing Hilbert transformation of channel', ichan

c...perform fft of i/p series (after extending series length by adding zeroes)
        do i = 1, npt
          cr = data(ichan,i)
          if( cr.eq.MISDAT ) cr = crl
          cdata( i ) = cmplx( cr, 0.0 )
          crl = cr
        enddo
        do i = npt+1, nptfft
          cdata( i ) = cmplx( 0.0, 0.0 )
        enddo
        call cfftf( nptfft, cdata, wsave )
        write(*,*)'Forward FFT performed'        

c...set d.c. & nyquist components to zero
        cdata( 1 )        = cmplx( 0.0, 0.0 )
        cdata( nptfft )   = cmplx( 0.0, 0.0 )
        cdata( nptfft/2 ) = cmplx( 0.0, 0.0 )

c...multiply by i*sgn
        do i = 1, nptfft/2
          cr =  real( cdata(i))
          ci = aimag( cdata(i))
          cdata(i) = cmplx( -ci,  cr )
        enddo
        do i = nptfft/2 +1, nptfft
          cr =  real( cdata(i))
          ci = aimag( cdata(i))
          cdata(i) = cmplx(  ci, -cr )
        enddo

c...perform ifft
        call cfftb(  nptfft, cdata, wsave )
        write(*,*)'Inverse FFT performed'

c...take real part alone of ifft and compute instataneous amplitude

        do i = 1, npt
          crl = data(ichan,i)
          if( crl.eq.MISDAT ) then
            datout(ichan,i) = MISDAT
          else
            cr =  real(cdata(i))/float(nptfft)
            datout(ichan,i) = sqrt( cr*cr + crl*crl )
          endif
        enddo

      enddo


c-------------------------------------------------------------------------------

      do i = 1, npt
        cr          = datout(3,i)
        datout(3,i) = datout(3,i)/sqrt(datout(1,i)*datout(1,i) +
     &                                 datout(2,i)*datout(2,i) )
        datout(1,i) = data(3,i)
        datout(2,i) = cr
      enddo

c-------------------------------------------------------------------------------
c...write data out

      form   = formo
      format = formato
      call ts_write( outd1, npt, datout )

      stop


9999  stop
      end
