c
c
c   x2edi
c
c------------------------------------------------------------------------------
c
c  Modification History
c
c  1.1: 30-May-94:agj: EX & Ey pot positions: S & W pots first respectively
c  1.2  Sept 2/94:jim: declination handled incorrectly - corrected
c
c  4.0  June 1996: changed to version 4.0
c                  .got rid of need for flat file (xxxsite.flt).
c
c------------------------------------------------------------------------------

      program x2edi
      
      implicit none
      
      include 'maxf.inc'
      integer MAXCHAN
      parameter(MAXCHAN=7)
      integer MAXS
      parameter(MAXS=200)

      include 'ctrlblk.inc'
      include 'io.inc'
      
      logical rr

      integer i, j, k, ii, jj
      integer narg, lbuff, ibuff, ledif
      integer ixpwr, iedi, iincl, linclf
      integer lelev, lreflat, lreflong
      integer idum,nper,nchan
      integer ispec_opt, ihx_az, ihy_az
      integer ia(3), idecl, sitenum
      integer iexlen, ieylen, iexaz, ieyaz
      integer x, x2, y, y2
      
      integer latdeg, latmin, latsec, londeg, lonmin, lonsec
      character*1 lat_hemi, lon_hemi, ex_unit, ey_unit,
     &     az_ref
      character*2 cprint, lat_deg, lat_min, lat_sec,
     &             lon_min, lon_sec
      character*3 lon_deg
     
      character*3 project, site, 
     &     ex_len, ey_len, ex_az, ey_az, hx_az, hy_az
      character*5 celev
      character*6 stat,crotspec_angle
      character*20 stat_info, reflat, reflong
      character*80 xpwrf, edif, buff, inclf, line

      real rotspec, exlen, eylen, exaz, eyaz, angle
      real period(MAXF), ndeg(MAXF)
      real packed_s(MAXCHAN,MAXCHAN)
      real lat, long, elev
      
      real cosd, sind

      character*24 date_string
      
      complex s7(MAXCHAN,MAXCHAN,MAXF), swap

      integer iargc

      logical PARAMS


c-------------------------------------------------------------------------------
c     get date

      call fdate(date_string)

c=====================================================================
c     initialize common

      write(*,*) ' '
      write(*,*) '                      x2edi'
      write(*,*) ' '
      write(*,*) 'NEW FEATURE: reads in params from x2edi.cfg file '
      write(*,*) '             will create it after first use'
      write(*,*) ' '

      narg = iargc()
      if( narg.ge.1 ) then
        call getarg( 1, cprint)
        read(cprint,'(i2)') iprint
      else
        iprint = 0
      end if
      call iin( 'Set error message print level (-1/0/1/2)', iprint)
c
c---check for existence of x2edi.cfg file
c

      open(unit=ind4,file='x2edi.cfg',status='old',err=1)
      write(*,*)'Opened file of parameters >x2edi.cfg'
      PARAMS = .TRUE.
      goto 2
1     write(*,*)'Parameters file not found, opening >x2edi.cfg',
     &          '   to write params into'
      open(unit=outd2,file='x2edi.cfg',status='new')
      PARAMS = .FALSE.

C
C     --- get project name 
C
   2  if( PARAMS ) then
        read(ind4,'(a)') project
      else
        project = 'tbt'
        call tin ( 'Give project code', project )
      endif

      sitenum = 101
      call iin('Enter 3-numeric site code', sitenum)
      write(site,'(i3.3)') sitenum

      stat = project//site

c     --- open cross-power file
      ixpwr = ind2
 200  xpwrf = stat//'/'//stat//'.xl'
      call iofile (2,'X-pwr file name?', ixpwr, xpwrf, *200 )
      if(index(xpwrf,'.xl').ne.0) then
         RR = .FALSE.
      else
         RR = .TRUE.
      endif

c...read over comment lines
      read(ixpwr,'(a)') line
      do while( line(1:1).eq.'#' )
        read(ixpwr,'(a)') line
      enddo

c...read in parameters
      do while( line(1:1).eq.'>' )
        if( line(2:11).eq.'STATION   ' ) then
          if( line(13:18).ne.stat ) then
            write(*,*)'file/station mismatch'
            write(*,*)'stat from filename =', stat
            write(*,*)'stat from file     =', line(13:18)
            write(*,*)'fix.......'
            stop
          endif
          stat = line(13:18)
        else if( line(2:11).eq.'LATITUDE  ' ) then
          read( line(13:),* ) lat
        else if( line(2:11).eq.'LONGITUDE ' ) then
          read( line(13:),* ) long
        else if( line(2:11).eq.'ELEVATION ' ) then
          read( line(13:),* ) elev
        endif

        read(ixpwr,'(a)',err=10) line
      enddo


c     --- the site has been located in .flt
  10  stat_info = ' '

      latdeg = ifix(lat)  
      latmin = ifix(60.*(lat - float(latdeg)))
      latsec = ifix(60.*(60.*(lat - float(latdeg)) - float(latmin)))
      write(lat_deg,'(i2.2)') latdeg
      write(lat_min,'(i2.2)') latmin
      write(lat_sec,'(i2.2)') latsec
      lat_hemi = 'N'
      
      londeg = ifix(long)
      lonmin = ifix(60.*(long - float(londeg)))
      lonsec = ifix(60.*(60.*(long - float(londeg)) - float(lonmin)))
      write(lon_deg,'(i3.3)') londeg
      write(lon_min,'(i2.2)') lonmin
      write(lon_sec,'(i2.2)') lonsec
      lon_hemi = 'E'
      
      ex_len = '1'
      ex_unit = '1'
      ey_len = '1'
      ey_unit = '1'
      az_ref = 'N'
      ex_az = '0'
      ey_az = '90'
      hx_az = '0'

c     --- read xpwr info, note that s7 requires some column and
c         row swapping below


c
c     --- note the strange arrangement of channels under
c         Jones (ie coming out of restack7)
c      Channel     LIMS        Cascade      Jones
c         1         Hx           Ex           Hx
c         2         Hy           Ey           Hy
c         3         Hz           Hx           Ex
c         4         Ex           Hy           Ey
c         5         Ey           Hz           Hz
c         6         Rx           Rx           Rx
c         7         Ry           Ry           Ry
c
c     --- read the lower triangle of the spectral matrix
c         in channel order givne by Jones column above
c
      nper = 0
      do k = 1, 1000
        write(*,*)'Reading in period.....',nper+1
        read(ixpwr,*,end=100) period(k), ndeg(k)
        do jj = 1, 7
          do ii = j, 7
            read(ixpwr,*,end=9999) idum, i, j, s7(i,j,k)
c           unpack by putting complex conjugate in upper triangle
            if(i.ne.j) then
               s7(j,i,k) = conjg(s7(i,j,k))
            endif
          end do
        end do
        nper = nper+1
      end do
100   close(ixpwr)
      write(*,*)'Read in file with nper found =', nper
c     
c     --- Geotools reads in spectral matrices of the type
c         under columns LIMS above; some row and column
c         swapping are required
c
c     --- swap columns 3 & 5 (Ex* and Hz*)
      do k =1, nper
         do i=1,7
            swap = s7(i,3,k)
            s7(i,3,k) = s7(i,5,k)
            s7(i,5,k) = swap
         end do
      end do
c     --- swap columns 4 & 5 (Ey* & Ex*)
      do k =1, nper
         do i=1,7
            swap = s7(i,4,k)
            s7(i,4,k) = s7(i,5,k)
            s7(i,5,k) = swap
         end do
      end do
c
c     ---swap rows 3 & 5 (Ex & Hz)
      do k =1, nper
         do j=1,7
            swap = s7(3,j,k)
            s7(3,j,k) = s7(5,j,k)
            s7(5,j,k) = swap
         end do
      end do
c
c     ---swap rows 4 & 5 (Ey & Hz)
      do k =1, nper
         do j=1,7
            swap = s7(4,j,k)
            s7(4,j,k) = s7(5,j,k)
            s7(5,j,k) = swap
         end do
      end do
c
c     ---- perform conjugation to get spectra into GT+ 
      if (iprint.gt.0) write(out,*)' Performing conjugation'
      do k = 1, nper
         do j = 1, 7
            do i = 1,7
               if (i.ne.j) s7(i,j,k) = conjg(s7(i,j,k))
            end do
         end do
      end do
C
C     --- open edifile
C     
      write(out,*)' '
      write(out,*)' Header and spectra successfully slurped in...'
      iedi = outd1
 300  edif = stat // '/' // stat // '.edi'
      call iofile(1,'Specify output edi filename', iedi, edif, *300)
      call trunc(edif,ledif)
c
c     --- HEAD
      write(iedi,*)'>HEAD'
      write(iedi,*)'    DATAID=',stat

      if( PARAMS ) then
        read(ind4,'(a)') buff
        call trunc(buff,lbuff)
      else
        buff = '"GSC/SEM MT TEAM"'
        call tin('Who collected the data?', buff)
        call trunc(buff,lbuff)
        write(outd2,*) buff(1:lbuff)
      endif
      write(iedi,*)'    ACQBY=',buff(:lbuff)

      if( PARAMS ) then
        read(ind4,'(a)') buff
        call trunc(buff,lbuff)
      else
        buff = '"GSC/SEM MT TEAM"'
        call tin('Who is writing this edi file?', buff)
        call trunc(buff,lbuff)
        write(outd2,*) buff(1:lbuff)
      endif
      write(iedi,*)'    FILEBY=',buff(:lbuff)

      if( PARAMS ) then
        read(ind4,'(a)') buff
        call trunc(buff,lbuff)
      else
        buff = '02/28/93'
        call tin('Specify an acquisition day (MM/DD/YY)?',buff)
        call trunc(buff,lbuff)
        write(outd2,*) buff(1:lbuff)
      endif
      write(iedi,*)'    ACQDATE=',buff(:lbuff)

      buff = '00/00/00'
      call idate(ia)
      write(buff(1:2),'(i2.2)') ia(2)
      write(buff(4:5),'(i2.2)') ia(1)
      write(buff(7:8),'(i2.2)') ia(3)-1900
      write(iedi,*)'    FILEDATE=',buff(:8)

      if( PARAMS ) then
        read(ind4,'(a)') buff
        call trunc(buff,lbuff)
      else
        buff = '"Canada"'
        call tin('Which country were the data collected in?',buff)
        call trunc(buff,lbuff)
        write(outd2,*) buff(1:lbuff)
      endif
      write(iedi,*)'    COUNTRY=',buff(:lbuff)

      if( PARAMS ) then
        read(ind4,'(a)') buff
        call trunc(buff,lbuff)
      else
        buff = '"Alberta"'
        call tin('Specify a province or state',buff)
        call trunc(buff,lbuff)
        write(outd2,*) buff(1:lbuff)
      endif
      write(iedi,*)'    STATE=',buff(:lbuff)

      if(lat_hemi.eq.'S') then
         reflat = 'LAT=-'//lat_deg//':'//lat_min//':'//lat_sec
      else
         reflat = 'LAT='//lat_deg//':'//lat_min//':'//lat_sec
      endif
      call trunc(reflat,lreflat)
      write(iedi,*)'    ', reflat(:lreflat)
      if(lon_hemi.eq.'W') then
         reflong = 'LONG=-'//lon_deg//':'//lon_min//':'//lon_sec
      else
         reflong = 'LONG='//lon_deg//':'//lon_min//':'//lon_sec
      endif
      call trunc(reflong,lreflong)
      write(iedi,*) '    ',reflong(:lreflong)

      ibuff = 500
      call iin('Elevation (metres)?', ibuff)
      write(celev,'(i5.5)') ibuff
      call trunc(celev,lelev) 
      write(iedi,*)'    ELEV=',celev(:lelev)
      write(iedi,*)'    UNITS=M'
      write(iedi,*)'    STDVERS="SEG 1.0"'
      write(iedi,*)'    PROGVERS="X2EDI 1.1"'
      write(iedi,*)'    PROGDATE=05/17/93'
      write(iedi,*)'    MAXSECT=99'
      write(iedi,*)'    EMPTY=1.0E+32'

c     --- INFO
      write(iedi,*)' '
      write(iedi,*)'>INFO MAXINFO=4500'
      write(iedi,*)stat_info
      
      iincl = ind3
 400  inclf = stat//'/'//stat//'.log'
      call tin('File to include in INFO section (xx : none)', inclf)
      call trunc(inclf,linclf)
      if(inclf(1:2).ne.'xx'.and.inclf(1:2).ne.'XX') then
         open(iincl,file=inclf,status='old',err=410)
         goto 450
      else 
         goto 500
      endif
 410  continue
      write(out,*)' Bad file name: ', inclf(:linclf)
      goto 400

 450  continue
        read(iincl,'(a80)',end=475) buff
        write(iedi,'(a80)') buff
        goto 450
 475  continue
      close(iincl)
c
c     --- DEFINEMEAS
 500  continue
      write(iedi,*)' '
      write(iedi,*)'>=DEFINEMEAS'
      if(rr) then
         write(iedi,*)'    MAXCHAN=7'
      else
         write(iedi,*)'    MAXCHAN=5'
      endif
      write(iedi,*)'    MAXRUN=999'
      if(rr) then
         write(iedi,*)'    MAXMEAS=7'
      else
         write(iedi,*)'    MAXMEAS=5'
      endif
      write(iedi,*)'    UNITS=M'
      write(iedi,*)'    REFTYPE=CART'
      write(iedi,*)'    REFLOC='//stat
      write(iedi,*)'    REF'//reflat(:lreflat)
      write(iedi,*)'    REF'//reflong(:lreflong)
      write(iedi,*)'    REFELEV='//celev(:lelev)
      write(iedi,*)' '

      if(az_ref.eq.'M') then
         write(out,*)' The site file reports sensor azimuths'
     &        //' relative to magnetic north.'
         idecl = -13
         call iin('What is the magnetic declination?', idecl)
         read(hx_az,'(i3)') ihx_az
c v1.2 (was - idecl)
         ihx_az = ihx_az + idecl
         write(hx_az,'(i3.3)') ihx_az
         read(ex_az,'(i3)') iexaz
c v1.2 (was - idecl)
         iexaz = iexaz + idecl
         write(ex_az,'(i3.3)') iexaz
         read(ey_az,'(i3)') ieyaz
c v1.2 (was - idecl)
         ieyaz = ieyaz + idecl
         write(ey_az,'(i3.3)') ieyaz
      endif
      write(iedi,*)'>HMEAS ID='//site//'1.001' //
     &     ' CHTYPE=HX X=8.5 Y=8.5 AZM='// hx_az //' ACQCHAN=CH1'
      read(hx_az,'(i3)') ihx_az
      ihy_az = ihx_az+90
      write(hy_az,'(i3.3)') ihy_az
      write(iedi,*)'>HMEAS ID='//site//'2.001' //
     &     ' CHTYPE=HY X=8.5 Y=8.5 AZM='// hy_az //' ACQCHAN=CH2'
      write(iedi,*)'>HMEAS ID='//site//'3.001' //
     &     ' CHTYPE=HZ X=8.5 Y=8.5 AZM=0 ACQCHAN=CH3'
      read(ex_len,'(i3)') iexlen
      read(ey_len,'(i3)') ieylen
      exlen = iexlen
      eylen = ieylen
      read(ex_az,'(i3)') iexaz
      read(ey_az,'(i3)') ieyaz
      exaz = iexaz
      eyaz = ieyaz
c
c---mod 1.1 made here
c      x = nint(exlen/2.*cosd(exaz))
      x = -nint(exlen/2.*cosd(exaz))
      x2 = -x
c      y = nint(exlen/2.*sind(exaz))
      y = -nint(exlen/2.*sind(exaz))
      y2 = -y

      write(iedi,*)'>EMEAS ID='//site//'4.001' //
     &     ' CHTYPE=EX X=',x,' Y=',y,' X2=',x2,' Y2=',y2,
     &     ' ACQCHAN=CH4'
      x = nint(eylen/2.*cosd(eyaz))
      x2 = -x
      y = nint(eylen/2.*sind(eyaz))
      y2 = -y
      write(iedi,*)'>EMEAS ID='//site//'5.001' //
     &     ' CHTYPE=EY X=',x,' Y=',y,' X2=',x2,' Y2=',y2,
     &     ' ACQCHAN=CH5'
      if(rr) then
        write(iedi,*)'>HMEAS ID='//site//'6.001' //
     &        ' CHTYPE=HX X=1000 Y=1000 AZM='// hx_az //' ACQCHAN=CH6'
        write(iedi,*)'>HMEAS ID='//site//'7.001' //
     &       ' CHTYPE=HY X=1000 Y=1000 AZM='// hy_az //' ACQCHAN=CH7'
      endif
c
C     --- SPECTRASECT
      write(iedi,*)' '
      write(iedi,*)'>=SPECTRASECT'
      write(iedi,*)'   SECTID="'//stat//'"'
      if(rr) then
         write(iedi,*)'   NCHAN=7'
      else
         write(iedi,*)'   NCHAN=5'
      endif
      write(iedi,*)'   NFREQ=',nper
      write(iedi,*)'   MAXBLKS=',nper
      if(rr) then
         write(iedi,*)'    // 7'
      else
         write(iedi,*)'    // 5'
      endif
      write(iedi,*)'    '//site//'1.001'
      write(iedi,*)'    '//site//'2.001'
      write(iedi,*)'    '//site//'3.001'
      write(iedi,*)'    '//site//'4.001'
      write(iedi,*)'    '//site//'5.001'
      if(rr) then
         write(iedi,*)'    '//site//'6.001'
         write(iedi,*)'    '//site//'7.001'
      endif

 600  rotspec = angle
      write(crotspec_angle,'(f6.1)')rotspec
      write(out,*)' '
      write(out,*)' Specify the X-dir of the spectra using one of the'
     &     //' options below.'
      write(out,*)' The angle specified must be relative to true'
     &     //' north.'
      write(out,*)' The angle is used by x2edi as a header entry only.'
      write(out,*)' x2edi does not rotate spectra.'
      write(out,*)' '
      write(out,*)' EDI spectra rotation (header entry) options:'
      write(out,*)'  1) omit ROTSPEC entry, Geotools will compute the'
     &     // ' rotation using Ex'
      write(out,*)'     pot positions'
      write(out,*)'  2) use ROTSPEC =' // crotspec_angle // ' degrees'
     &     //' from True North'
      write(out,*)'  3) enter a value for ROTSPEC'
      ispec_opt = 2
      call iin('Enter an integer', ispec_opt)
      if(ispec_opt.lt.1.or.ispec_opt.gt.3 ) then
         write(out,*)' Enter a number between 1 & 3'
         goto 600
      endif
      if (ispec_opt.eq.3 ) then
         call rin('Enter ROTSPEC angle (w.r.t. TRUE NORTH)',
     &        rotspec)
         write(crotspec_angle,'(f6.1)')rotspec
      endif

      write(iedi,*) ' '
      if (rr ) then
         nchan = 7
      else
         nchan = 5
      end if
      do k=1,nper
c        --- place imaginary part in upper triangle of packed spectra
c            (we should preserve the sign of the imaginary part according to 
c             SEG EDI specification, but we must conjugate for an unknown 
c             reason)
c        --- put real part in lower triangle
         do i = 1, nchan
            do j = i+1, nchan
               packed_s(i,j)=-1.*aimag(s7(i,j,k))
c               write(*,*) 'u', i,j, s7(i,j,k), packed_s(i,j)
               packed_s(j,i) = real(s7(i,j,k))
c               write(*,*) 'l', j,i, s7(i,j,k), packed_s(j,i)
            end do
         end do
c        --- auto powers along the diagonal
         do i=1,nchan
            j=i
            packed_s(i,j) = real(s7(i,j,k))
c            write(*,*) 'd', i,j, s7(i,j,k), packed_s(i,j)
         end do
c        --- write out header for this freq est.
 700     format(a,1pe10.3,a,a6,a,1pe10.3,a,1pe10.3,a)
 710     format(a,1pe10.3,a,1pe10.3,a,1pe10.3,a)
         if(rr) then
            if(ispec_opt.ne.1) then
               write(iedi,700)'>SPECTRA FREQ=',1./period(k),
     &              ' ROTSPEC=',crotspec_angle,' AVGT=',ndeg(k),
     &              ' BW=',0.25/period(k),
     &              ' // 49'
            else
               write(iedi,710)'>SPECTRA FREQ=',1./period(k),
     &              ' BW=',0.25/period(k),
     &              ' AVGT=',ndeg(k),' // 49'
            endif
         else
            if(ispec_opt.ne.1) then
               write(iedi,700)'>SPECTRA FREQ=',1./period(k),
     &              ' ROTSPEC=',crotspec_angle,' AVGT=',ndeg(k),
     &              ' BW=',0.25/period(k),
     &              ' // 25'
            else
               write(iedi,710)'>SPECTRA FREQ=',1./period(k),
     &              ' BW=',0.25/period(k),
     &              ' AVGT=',ndeg(k),' // 25'
            endif
         endif
c        --- write out the spectra block
         do i=1, nchan
            write(iedi,'(7(1pe15.8,1x))')(packed_s(i,j),j=1,nchan)
         end do
      end do
      write(iedi,*)' '
      write(iedi,*)'>END'
      close(iedi)
      write(out,*)' File ', edif(:ledif),' successfully written.'

      stop

 9999 continue
      stop
      end
